@@ -1,5 +1,5 @@
 from __future__ import print_function
-import sys, shutil, os, tempfile, hashlib, time
+import sys, shutil, os, tempfile, hashlib
 import sysconfig
 from os.path import join
 
@@ -172,15 +172,16 @@ def create_cffi_import_libraries(pypy_c, options, basedir, only=None,
 
     shutil.rmtree(str(join(basedir,'lib_pypy','__pycache__')),
                   ignore_errors=True)
-    # be sure pip, setuptools are installed in a fresh pypy
-    # allows proper functioning of cffi on win32 with newer vc compilers
-    # XXX move this to a build slave step?
     env = os.environ
     if sys.platform == 'win32':
+        # requires the repo pypy/externals to be located under pypy
+        externals_path = os.path.abspath(os.path.join(os.path.dirname(__file__),
+                                        '..', '..', 'externals'))
+        assert os.path.exists(externals_path), externals_path
         env = os.environ.copy()
-        env['INCLUDE'] = r'..\externals\include;' + env.get('INCLUDE', '')
-        env['LIB'] = r'..\externals\lib;' + env.get('LIB', '')
-        env['PATH'] = r'..\externals\bin;' + env.get('PATH', '')
+        env['INCLUDE'] = externals_path + r'\include;' + env.get('INCLUDE', '')
+        env['LIB'] = externals_path + r'\lib;' + env.get('LIB', '')
+        env['PATH'] = externals_path + r'\bin;' + env.get('PATH', '')
     status, stdout, stderr = run_subprocess(str(pypy_c), ['-c', 'import setuptools'])
     if status  != 0:
         status, stdout, stderr = run_subprocess(str(pypy_c), ['-m', 'ensurepip'])
