@@ -5,7 +5,7 @@ if hasattr(sys, 'setrecursionlimit'):
     # some tests fail otherwise
     sys.setrecursionlimit(2000)
 
-LOOK_FOR_PYTHON3 = 'python3.6'
+LOOK_FOR_PYTHON3 = 'python3.7'
 PYTHON3 = os.getenv('PYTHON3') or py.path.local.sysfind(LOOK_FOR_PYTHON3)
 if PYTHON3 is not None:
     PYTHON3 = str(PYTHON3)
@@ -68,6 +68,11 @@ def pytest_configure(config):
     if not mode_A and not mode_D:  # 'own' tests
         from rpython.conftest import LeakFinder
         config.pluginmanager.register(LeakFinder())
+    if mode_A:
+        from pypy.tool.pytest.apptest import PythonInterpreter
+        config.applevel = PythonInterpreter(config.option.python)
+    else:
+        config.applevel = None
 
 def pytest_addoption(parser):
     group = parser.getgroup("pypy options")
@@ -79,7 +84,7 @@ def pytest_addoption(parser):
            help="python interpreter to run appdirect tests with")
     group.addoption('-D', '--direct-apptest', action="store_true",
            default=False, dest="direct_apptest",
-           help="run applevel_XXX.py tests directly on host interpreter")
+           help="run '%s' tests directly on host interpreter" % APPLEVEL_FN)
     group.addoption('--direct', action="store_true",
            default=False, dest="rundirect",
            help="run pexpect tests directly")
@@ -135,8 +140,8 @@ def pytest_pycollect_makemodule(path, parent):
         return PyPyModule(path, parent)
 
 def is_applevel(item):
-    from pypy.tool.pytest.apptest import AppTestFunction
-    return isinstance(item, AppTestFunction)
+    from pypy.tool.pytest.apptest import AppTestMethod
+    return isinstance(item, AppTestMethod)
 
 def pytest_collection_modifyitems(config, items):
     if config.getoption('runappdirect') or config.getoption('direct_apptest'):
@@ -166,8 +171,6 @@ class PyPyModule(pytest.Module):
     def funcnamefilter(self, name):
         if name.startswith('test_'):
             return self.accept_regular_test()
-        if name.startswith('app_test_'):
-            return True
         return False
 
     def classnamefilter(self, name):
@@ -182,13 +185,6 @@ class PyPyModule(pytest.Module):
             if name.startswith('AppTest'):
                 from pypy.tool.pytest.apptest import AppClassCollector
                 return AppClassCollector(name, parent=self)
-
-        elif hasattr(obj, 'func_code') and self.funcnamefilter(name):
-            if name.startswith('app_test_'):
-                assert not obj.func_code.co_flags & 32, \
-                    "generator app level functions? you must be joking"
-                from pypy.tool.pytest.apptest import AppTestFunction
-                return AppTestFunction(name, parent=self)
         return super(PyPyModule, self).makeitem(name, obj)
 
 def skip_on_missing_buildoption(**ropts):
@@ -207,28 +203,20 @@ def skip_on_missing_buildoption(**ropts):
     py.test.skip("need translated pypy3 with: %s, got %s"
                  %(ropts,options))
 
-class LazyObjSpaceGetter(object):
-    def __get__(self, obj, cls=None):
-        from pypy.tool.pytest.objspace import gettestobjspace
-        space = gettestobjspace()
-        if cls:
-            cls.space = space
-        return space
-
-
 @pytest.hookimpl(tryfirst=True)
 def pytest_runtest_setup(item):
     if isinstance(item, py.test.collect.Function):
+        config = item.config
+        if item.get_marker(name='pypy_only'):
+            if config.applevel is not None and not config.applevel.is_pypy:
+                pytest.skip('PyPy-specific test')
         appclass = item.getparent(py.test.Class)
         if appclass is not None:
+            from pypy.tool.pytest.objspace import gettestobjspace
             # Make cls.space and cls.runappdirect available in tests.
-            spaceconfig = getattr(appclass.obj, 'spaceconfig', None)
-            if spaceconfig is not None:
-                from pypy.tool.pytest.objspace import gettestobjspace
-                appclass.obj.space = gettestobjspace(**spaceconfig)
-            else:
-                appclass.obj.space = LazyObjSpaceGetter()
-            appclass.obj.runappdirect = option.runappdirect
+            spaceconfig = getattr(appclass.obj, 'spaceconfig', {})
+            appclass.obj.space = gettestobjspace(**spaceconfig)
+            appclass.obj.runappdirect = config.option.runappdirect
 
 def pytest_ignore_collect(path, config):
     if (config.getoption('direct_apptest') and not path.isdir()
