@@ -32,12 +32,15 @@ Options and arguments (and corresponding environment variables):
 -W arg : warning control; arg is action:message:category:module:lineno
          also PYTHONWARNINGS=arg
 -X opt : set implementation-specific option
+--check-hash-based-pycs always|default|never:
+    control how Python invalidates hash-based .pyc files
 file   : program read from script file
 -      : program read from stdin (default; interactive mode if a tty)
 arg ...: arguments passed to program in sys.argv[1:]
 PyPy options and arguments:
 --info : print translation information about this PyPy executable
 -X faulthandler: attempt to display tracebacks when PyPy crashes
+-X dev: enable PyPy's "development mode"
 """
 # Missing vs CPython: PYTHONHOME
 USAGE2 = """
@@ -47,6 +50,8 @@ PYTHONPATH   : %r-separated list of directories prefixed to the
                default module search path.  The result is sys.path.
 PYTHONCASEOK : ignore case in 'import' statements (Windows).
 PYTHONIOENCODING: Encoding[:errors] used for stdin/stdout/stderr.
+PYTHONFAULTHANDLER: dump the Python traceback on fatal errors.
+PYTHONDEVMODE: enable the development mode.
 PYPY_IRC_TOPIC: if set to a non-empty value, print a random #pypy IRC
                topic at startup of interactive mode.
 PYPYLOG: If set to a non-empty value, enable logging.
@@ -64,6 +69,8 @@ except ImportError:
 import errno
 import sys
 
+_MACOSX = sys.platform == 'darwin'
+
 DEBUG = False       # dump exceptions before calling the except hook
 
 originalexcepthook = sys.__excepthook__
@@ -270,10 +277,7 @@ def fdopen(fd, mode, bufsize=-1):
 # ____________________________________________________________
 # Main entry point
 
-def we_are_translated():
-    # app-level, very different from rpython.rlib.objectmodel.we_are_translated
-    return hasattr(sys, 'pypy_translation_info')
-
+WE_ARE_TRANSLATED = True   # patch to False if we're not really translated
 IS_WINDOWS = 'nt' in sys.builtin_module_names
 
 def setup_and_fix_paths(ignore_environment=False, **extra):
@@ -384,6 +388,8 @@ sys_flags = (
     "quiet",
     "hash_randomization",
     "isolated",
+    "dev_mode",
+    "utf8_mode",
 )
 # ^^^ Order is significant!  Keep in sync with module.sys.app.sysflags
 
@@ -394,6 +400,8 @@ default_options = dict.fromkeys(
     "run_stdin",
     "warnoptions",
     "unbuffered"), 0)
+default_options["check_hash_based_pycs"] = "default"
+default_options["dev_mode"] = False # needs to be bool
 
 def simple_option(options, name, iterargv):
     options[name] += 1
@@ -413,6 +421,8 @@ def m_option(options, runmodule, iterargv):
 
 def X_option(options, xoption, iterargv):
     options["_xoptions"].append(xoption)
+    if xoption == "dev":
+        options["dev_mode"] = True
 
 def W_option(options, warnoption, iterargv):
     options["warnoptions"].append(warnoption)
@@ -423,6 +433,16 @@ def end_options(options, _, iterargv):
 def ignore_option(*args):
     pass
 
+def check_hash_based_pycs(options, value, iterargv):
+    if value not in ("default", "always", "never"):
+        initstdio()
+        print_error("--check-hash-based-pycs must be one of 'default', 'always', or 'never'")
+        raise SystemExit
+    import _imp
+    _imp.check_hash_based_pycs = value
+    options["check_hash_based_pycs"] = value
+
+
 cmdline_options = {
     # simple options just increment the counter of the options listed above
     'b': (simple_option, 'bytes_warning'),
@@ -450,6 +470,7 @@ cmdline_options = {
     '--info':    (print_info,      None),
     '--jit':     (set_jit_option,  Ellipsis),
     '-funroll-loops': (funroll_loops, None),
+    '--check-hash-based-pycs': (check_hash_based_pycs, Ellipsis),
     '--':        (end_options,     None),
     'R':         (ignore_option,   None),      # previously hash_randomization
     }
@@ -538,11 +559,13 @@ def parse_command_line(argv):
             options["unbuffered"] = 1
         parse_env('PYTHONVERBOSE', "verbose", options)
         parse_env('PYTHONOPTIMIZE', "optimize", options)
+        if os.getenv('PYTHONDEVMODE'):
+            options["dev_mode"] = True
     if (options["interactive"] or
         (not options["ignore_environment"] and os.getenv('PYTHONINSPECT'))):
         options["inspect"] = 1
 
-    if we_are_translated():
+    if WE_ARE_TRANSLATED:
         flags = [options[flag] for flag in sys_flags]
         sys.flags = type(sys.flags)(flags)
         sys.dont_write_bytecode = bool(sys.flags.dont_write_bytecode)
@@ -550,7 +573,7 @@ def parse_command_line(argv):
     sys._xoptions = dict(x.split('=', 1) if '=' in x else (x, True)
                          for x in options['_xoptions'])
 
-##    if not we_are_translated():
+##    if not WE_ARE_TRANSLATED:
 ##        for key in sorted(options):
 ##            print '%40s: %s' % (key, options[key])
 ##        print '%40s: %s' % ("sys.argv", sys.argv)
@@ -567,22 +590,25 @@ def run_command_line(interactive,
                      warnoptions,
                      unbuffered,
                      ignore_environment,
-                     quiet,
                      verbose,
+                     bytes_warning,
+                     quiet,
                      isolated,
+                     dev_mode,
                      **ignored):
     # with PyPy in top of CPython we can only have around 100
-    # but we need more in the translated PyPy for the compiler package
-    if '__pypy__' not in sys.builtin_module_names:
+    # but we need more in the PyPy level for the compiler package
+    if not WE_ARE_TRANSLATED:
         sys.setrecursionlimit(5000)
     import os
 
+
     readenv = not ignore_environment
     io_encoding = os.getenv("PYTHONIOENCODING") if readenv else None
     initstdio(io_encoding, unbuffered)
 
     if 'faulthandler' in sys.builtin_module_names:
-        if 'faulthandler' in sys._xoptions or os.getenv('PYTHONFAULTHANDLER'):
+        if dev_mode or 'faulthandler' in sys._xoptions or (readenv and os.getenv('PYTHONFAULTHANDLER')):
             import faulthandler
             try:
                 faulthandler.enable(2)   # manually set to stderr
@@ -596,16 +622,44 @@ def run_command_line(interactive,
     sys.modules['__main__'] = mainmodule
 
     if not no_site:
+        if _MACOSX:
+            # __PYVENV_LAUNCHER__, used by CPython on macOS, should be ignored
+            # since it (possibly) results in a wrong sys.prefix and
+            # sys.exec_prefix (and consequently sys.path) set by site.py.
+            old_pyvenv_launcher = os.environ.pop('__PYVENV_LAUNCHER__', None)
         try:
             import site
         except:
             print("'import site' failed", file=sys.stderr)
+        if _MACOSX and old_pyvenv_launcher:
+            os.environ['__PYVENV_LAUNCHER__'] = old_pyvenv_launcher
 
+    # The priority order for warnings configuration is (highest precedence
+    # first):
+    #
+    # - the BytesWarning filter, if needed ('-b', '-bb')
+    # - any '-W' command line options; then
+    # - the 'PYTHONWARNINGS' environment variable; then
+    # - the dev mode filter ('-X dev', 'PYTHONDEVMODE'); then
+    # - any implicit filters added by _warnings.c/warnings.py
+    #
+    # All settings except the last are passed to the warnings module via
+    # the `sys.warnoptions` list. Since the warnings module works on the basis
+    # of "the most recently added filter will be checked first", we add
+    # the lowest precedence entries first so that later entries override them.
+    sys_warnoptions = []
+    if dev_mode:
+        sys_warnoptions.append("default")
     pythonwarnings = readenv and os.getenv('PYTHONWARNINGS')
     if pythonwarnings:
-        warnoptions = pythonwarnings.split(',') + warnoptions
+        sys_warnoptions.extend(pythonwarnings.split(','))
     if warnoptions:
-        sys.warnoptions[:] = warnoptions
+        sys_warnoptions.extend(warnoptions)
+    if bytes_warning:
+        sys_warnoptions.append("error::BytesWarning" if bytes_warning > 1 else "default::BytesWarning")
+
+    if sys_warnoptions:
+        sys.warnoptions[:] = sys_warnoptions
         try:
             if 'warnings' in sys.modules:
                 from warnings import _processoptions
@@ -815,8 +869,7 @@ def run_command_line(interactive,
             if hasattr(sys, '__interactivehook__'):
                 run_toplevel(sys.__interactivehook__)
             pypy_version_info = getattr(sys, 'pypy_version_info', sys.version_info)
-            irc_topic = pypy_version_info[3] != 'final' or (
-                            readenv and os.getenv('PYPY_IRC_TOPIC'))
+            irc_topic = readenv and os.getenv('PYPY_IRC_TOPIC')
             success = run_toplevel(interactive_console, mainmodule,
                                    quiet=quiet or not irc_topic)
         except SystemExit as e:
@@ -933,6 +986,7 @@ if __name__ == '__main__':
         import os
         return os.path.abspath(os.path.join(s, '..'))
 
+    WE_ARE_TRANSLATED = False
     import os
     reset = []
     if 'PYTHONINSPECT_' in os.environ:
