@@ -2,13 +2,20 @@
 Arguments objects.
 """
 from rpython.rlib.debug import make_sure_not_resized
-from rpython.rlib.objectmodel import not_rpython
+from rpython.rlib.objectmodel import not_rpython, specialize
 from rpython.rlib import jit
 from rpython.rlib.objectmodel import enforceargs
 from rpython.rlib.rstring import StringBuilder
 
 from pypy.interpreter.error import OperationError, oefmt
 
+@specialize.arg(2)
+def raise_type_error(space, fnname_parens, msg, *args):
+    if fnname_parens is None:
+        raise oefmt(space.w_TypeError, msg, *args)
+    msg = "%s " + msg
+    raise oefmt(space.w_TypeError, msg, fnname_parens, *args)
+
 
 class Arguments(object):
     """
@@ -21,11 +28,9 @@ class Arguments(object):
     semantics are complex, but calls occur everywhere.
     """
 
-    ###  Construction  ###
-    #@enforceargs(keywords=[unicode])
     def __init__(self, space, args_w, keywords=None, keywords_w=None,
                  w_stararg=None, w_starstararg=None, keyword_names_w=None,
-                 methodcall=False):
+                 methodcall=False, fnname_parens=None):
         self.space = space
         assert isinstance(args_w, list)
         self.arguments_w = args_w
@@ -41,7 +46,7 @@ class Arguments(object):
             make_sure_not_resized(self.keywords_w)
 
         make_sure_not_resized(self.arguments_w)
-        self._combine_wrapped(w_stararg, w_starstararg)
+        self._combine_wrapped(w_stararg, w_starstararg, fnname_parens)
         # a flag that specifies whether the JIT can unroll loops that operate
         # on the keywords
         self._jit_few_keywords = self.keywords is None or jit.isconstant(len(self.keywords))
@@ -79,14 +84,14 @@ class Arguments(object):
         "Return a new Arguments with a new argument inserted first."
         return self.replace_arguments([w_firstarg] + self.arguments_w)
 
-    def _combine_wrapped(self, w_stararg, w_starstararg):
+    def _combine_wrapped(self, w_stararg, w_starstararg, fnname_parens=None):
         "unpack the *arg and **kwd into arguments_w and keywords_w"
         if w_stararg is not None:
-            self._combine_starargs_wrapped(w_stararg)
+            self._combine_starargs_wrapped(w_stararg, fnname_parens)
         if w_starstararg is not None:
-            self._combine_starstarargs_wrapped(w_starstararg)
+            self._combine_starstarargs_wrapped(w_starstararg, fnname_parens)
 
-    def _combine_starargs_wrapped(self, w_stararg):
+    def _combine_starargs_wrapped(self, w_stararg, fnname_parens=None):
         # unpack the * arguments
         space = self.space
         try:
@@ -94,13 +99,13 @@ class Arguments(object):
         except OperationError as e:
             if (e.match(space, space.w_TypeError) and
                     not space.is_iterable(w_stararg)):
-                raise oefmt(space.w_TypeError,
+                raise_type_error(space, fnname_parens,
                             "argument after * must be an iterable, not %T",
                             w_stararg)
             raise
         self.arguments_w = self.arguments_w + args_w
 
-    def _combine_starstarargs_wrapped(self, w_starstararg):
+    def _combine_starstarargs_wrapped(self, w_starstararg, fnname_parens=None):
         # unpack the ** arguments
         space = self.space
         keywords, values_w = space.view_as_kwargs(w_starstararg)
@@ -110,7 +115,8 @@ class Arguments(object):
                 self.keywords_w = values_w
             else:
                 _check_not_duplicate_kwargs(
-                    self.space, self.keywords, keywords, values_w)
+                    self.space, self.keywords, keywords, values_w,
+                    fnname_parens)
                 self.keywords = self.keywords + keywords
                 self.keywords_w = self.keywords_w + values_w
             return
@@ -123,7 +129,7 @@ class Arguments(object):
                 w_keys = space.call_method(w_starstararg, "keys")
             except OperationError as e:
                 if e.match(space, space.w_AttributeError):
-                    raise oefmt(space.w_TypeError,
+                    raise_type_error(space, fnname_parens,
                                 "argument after ** must be a mapping, not %T",
                                 w_starstararg)
                 raise
@@ -132,7 +138,7 @@ class Arguments(object):
         keywords = [None] * len(keys_w)
         _do_combine_starstarargs_wrapped(
             space, keys_w, w_starstararg, keywords, keywords_w, self.keywords,
-            is_dict)
+            is_dict, fnname_parens)
         self.keyword_names_w = keys_w
         if self.keywords is None:
             self.keywords = keywords
@@ -175,19 +181,23 @@ class Arguments(object):
         # some comments about the JIT: it assumes that signature is a constant,
         # so all values coming from there can be assumed constant. It assumes
         # that the length of the defaults_w does not vary too much.
+        co_posonlyargcount = signature.num_posonlyargnames()
         co_argcount = signature.num_argnames() # expected formal arguments, without */**
         co_kwonlyargcount = signature.num_kwonlyargnames()
         too_many_args = False
 
         # put the special w_firstarg into the scope, if it exists
-        if w_firstarg is not None:
-            upfront = 1
-            if co_argcount > 0:
-                scope_w[0] = w_firstarg
-        else:
-            upfront = 0
-
+        upfront = 0
         args_w = self.arguments_w
+        if w_firstarg is not None:
+            if co_posonlyargcount + co_argcount > 0:
+                scope_w[0] = w_firstarg
+                upfront = 1
+            else:
+                # ugh, this is a call to a method 'def meth(*args)', maybe
+                # (see test_issue2996_*).  Fall-back solution...
+                args_w = [w_firstarg] + args_w
+
         num_args = len(args_w)
         avail = num_args + upfront
 
@@ -198,36 +208,33 @@ class Arguments(object):
 
         # put as many positional input arguments into place as available
         input_argcount = upfront
-        if input_argcount < co_argcount:
-            take = min(num_args, co_argcount - upfront)
+        if input_argcount < co_posonlyargcount + co_argcount:
+            take = min(num_args, co_posonlyargcount + co_argcount - upfront)
 
             # letting the JIT unroll this loop is safe, because take is always
-            # smaller than co_argcount
+            # smaller than co_posonlyargcount + co_argcount
             for i in range(take):
                 scope_w[i + input_argcount] = args_w[i]
             input_argcount += take
 
         # collect extra positional arguments into the *vararg
         if signature.has_vararg():
-            args_left = co_argcount - upfront
-            if args_left < 0:  # check required by rpython
-                starargs_w = [w_firstarg]
-                if num_args:
-                    starargs_w = starargs_w + args_w
-            elif num_args > args_left:
+            args_left = co_posonlyargcount + co_argcount - upfront
+            assert args_left >= 0  # check required by rpython
+            if num_args > args_left:
                 starargs_w = args_w[args_left:]
             else:
                 starargs_w = []
-            loc = co_argcount + co_kwonlyargcount
+            loc = co_posonlyargcount + co_argcount + co_kwonlyargcount
             scope_w[loc] = self.space.newtuple(starargs_w)
-        elif avail > co_argcount:
+        elif avail > co_posonlyargcount + co_argcount:
             too_many_args = True
 
         # if a **kwargs argument is needed, create the dict
         w_kwds = None
         if signature.has_kwarg():
             w_kwds = self.space.newdict(kwargs=True)
-            scope_w[co_argcount + co_kwonlyargcount + signature.has_vararg()] = w_kwds
+            scope_w[co_posonlyargcount + co_argcount + co_kwonlyargcount + signature.has_vararg()] = w_kwds
 
         # handle keyword arguments
         num_remainingkwds = 0
@@ -236,7 +243,7 @@ class Arguments(object):
         if num_kwds:
             # kwds_mapping maps target indexes in the scope (minus input_argcount)
             # to positions in the keywords_w list
-            kwds_mapping = [0] * (co_argcount + co_kwonlyargcount - input_argcount)
+            kwds_mapping = [0] * (co_posonlyargcount + co_argcount + co_kwonlyargcount - input_argcount)
             # initialize manually, for the JIT :-(
             for i in range(len(kwds_mapping)):
                 kwds_mapping[i] = -1
@@ -245,8 +252,8 @@ class Arguments(object):
             # this function must not take a scope_w, to make the scope not
             # escape
             num_remainingkwds = _match_keywords(
-                    signature, blindargs, input_argcount, keywords,
-                    kwds_mapping, self._jit_few_keywords)
+                    signature, blindargs, co_posonlyargcount, input_argcount,
+                    keywords, kwds_mapping, self._jit_few_keywords)
             if num_remainingkwds:
                 if w_kwds is not None:
                     # collect extra keyword arguments into the **kwarg
@@ -261,14 +268,14 @@ class Arguments(object):
         # or with defaults, if available
         missing_positional = []
         missing_kwonly = []
-        more_filling = (input_argcount < co_argcount + co_kwonlyargcount)
+        more_filling = (input_argcount < co_posonlyargcount + co_argcount + co_kwonlyargcount)
         def_first = 0
         if more_filling:
-            def_first = co_argcount - (0 if defaults_w is None else len(defaults_w))
+            def_first = co_posonlyargcount + co_argcount - (0 if defaults_w is None else len(defaults_w))
             j = 0
             kwds_index = -1
             # first, fill the arguments from the kwds
-            for i in range(input_argcount, co_argcount + co_kwonlyargcount):
+            for i in range(input_argcount, co_posonlyargcount + co_argcount + co_kwonlyargcount):
                 if kwds_mapping is not None:
                     kwds_index = kwds_mapping[j]
                     j += 1
@@ -277,7 +284,7 @@ class Arguments(object):
 
         if too_many_args:
             kwonly_given = 0
-            for i in range(co_argcount, co_argcount + co_kwonlyargcount):
+            for i in range(co_posonlyargcount + co_argcount, co_posonlyargcount + co_argcount + co_kwonlyargcount):
                 if scope_w[i] is not None:
                     kwonly_given += 1
             if self.methodcall:
@@ -289,21 +296,23 @@ class Arguments(object):
                                 avail, kwonly_given)
 
         if more_filling:
-            # then, fill the normal arguments with defaults_w (if needed)
-            for i in range(input_argcount, co_argcount):
+            # then, fill the posonly arguments with defaults_w (if needed)
+            for i in range(input_argcount, co_posonlyargcount + co_argcount):
                 if scope_w[i] is not None:
                     continue
                 defnum = i - def_first
                 if defnum >= 0:
                     scope_w[i] = defaults_w[defnum]
+                elif i < co_posonlyargcount:
+                    missing_positional.append(signature.posonlyargnames[i])
                 else:
-                    missing_positional.append(signature.argnames[i])
+                    missing_positional.append(signature.argnames[i - co_posonlyargcount])
 
             # finally, fill kwonly arguments with w_kw_defs (if needed)
-            for i in range(co_argcount, co_argcount + co_kwonlyargcount):
+            for i in range(co_posonlyargcount + co_argcount, co_posonlyargcount + co_argcount + co_kwonlyargcount):
                 if scope_w[i] is not None:
                     continue
-                name = signature.kwonlyargnames[i - co_argcount]
+                name = signature.kwonlyargnames[i - (co_posonlyargcount + co_argcount)]
                 if w_kw_defs is None:
                     missing_kwonly.append(name)
                     continue
@@ -349,8 +358,8 @@ class Arguments(object):
     def parse_obj(self, w_firstarg,
                   fnname, signature, defaults_w=None, w_kw_defs=None,
                   blindargs=0):
-        """Parse args and kwargs to initialize a frame
-        according to the signature of code object.
+        """Parse args and kwargs into a list according to the signature of a
+        code object.
         """
         try:
             return self._parse(w_firstarg, signature, defaults_w, w_kw_defs,
@@ -387,28 +396,28 @@ class Arguments(object):
 # look at. They should not get a self arguments, which makes the amount of
 # arguments annoying :-(
 
-@jit.look_inside_iff(lambda space, existingkeywords, keywords, keywords_w:
+@jit.look_inside_iff(lambda space, existingkeywords, keywords, keywords_w, fnname_parens:
         jit.isconstant(len(keywords) and
         jit.isconstant(existingkeywords)))
-def _check_not_duplicate_kwargs(space, existingkeywords, keywords, keywords_w):
+def _check_not_duplicate_kwargs(space, existingkeywords, keywords, keywords_w, fnname_parens):
     # looks quadratic, but the JIT should remove all of it nicely.
     # Also, all the lists should be small
     for key in keywords:
         for otherkey in existingkeywords:
             if otherkey == key:
-                raise oefmt(space.w_TypeError,
+                raise_type_error(space, fnname_parens,
                             "got multiple values for keyword argument '%s'",
                             key)
 
 def _do_combine_starstarargs_wrapped(space, keys_w, w_starstararg, keywords,
-        keywords_w, existingkeywords, is_dict):
+        keywords_w, existingkeywords, is_dict, fnname_parens):
     i = 0
     for w_key in keys_w:
         try:
             key = space.text_w(w_key)
         except OperationError as e:
             if e.match(space, space.w_TypeError):
-                raise oefmt(space.w_TypeError,
+                raise_type_error(space, fnname_parens,
                             "keywords must be strings, not '%T'",
                             w_key)
             if e.match(space, space.w_UnicodeEncodeError):
@@ -418,7 +427,7 @@ def _do_combine_starstarargs_wrapped(space, keys_w, w_starstararg, keywords,
                 raise
         else:
             if existingkeywords and key in existingkeywords:
-                raise oefmt(space.w_TypeError,
+                raise_type_error(space, fnname_parens,
                             "got multiple values for keyword argument '%s'",
                             key)
         keywords[i] = key
@@ -435,10 +444,10 @@ def _do_combine_starstarargs_wrapped(space, keys_w, w_starstararg, keywords,
         i += 1
 
 @jit.look_inside_iff(
-    lambda signature, blindargs, input_argcount,
+    lambda signature, blindargs, co_posonlyargcount, input_argcount,
            keywords, kwds_mapping, jiton: jiton)
-def _match_keywords(signature, blindargs, input_argcount,
-                    keywords, kwds_mapping, _):
+def _match_keywords(signature, blindargs, co_posonlyargcount,
+                    input_argcount, keywords, kwds_mapping, _):
     # letting JIT unroll the loop is *only* safe if the callsite didn't
     # use **args because num_kwds can be arbitrarily large otherwise.
     num_kwds = num_remainingkwds = len(keywords)
@@ -449,9 +458,12 @@ def _match_keywords(signature, blindargs, input_argcount,
         if name is None:
             continue
         j = signature.find_argname(name)
-        # if j == -1 nothing happens, because j < input_argcount and
-        # blindargs > j
-        if j < input_argcount:
+        if 0 <= j < co_posonlyargcount:
+            raise ArgErrPosonlyAsKwds(signature.posonlyargnames[j])
+        elif j < input_argcount:
+            # if j == -1 nothing happens, because j < input_argcount and
+            # blindargs > j
+
             # check that no keyword argument conflicts with these. note
             # that for this purpose we ignore the first blindargs,
             # which were put into place by prepend().  This way,
@@ -530,7 +542,7 @@ class ArgErrTooMany(ArgErr):
         self.kwonly_given = kwonly_given
 
     def getmsg(self):
-        num_args = self.signature.num_argnames()
+        num_args = self.signature.num_posonlyargnames() + self.signature.num_argnames()
         num_defaults = self.num_defaults
         if num_defaults:
             takes_str = "from %d to %d positional arguments" % (
@@ -558,7 +570,7 @@ class ArgErrTooManyMethod(ArgErrTooMany):
 
     def getmsg(self):
         msg = ArgErrTooMany.getmsg(self)
-        n = self.signature.num_argnames()
+        n = self.signature.num_posonlyargnames() + self.signature.num_argnames()
         if (self.given == n + 1 and
                 (n == 0 or self.signature.argnames[0] != "self")):
             msg += ". Did you forget 'self' in the function definition?"
@@ -612,3 +624,16 @@ class ArgErrUnknownKwds(ArgErr):
             msg = "got %d unexpected keyword arguments" % (
                 self.num_kwds)
         return msg
+
+
+class ArgErrPosonlyAsKwds(ArgErr):
+
+    def __init__(self, posonly_kwd):
+        self.posonly_kwd = posonly_kwd
+
+    def getmsg(self):
+        # this message will have to be different in 3.8, where positional only
+        # arguments are a "real" concept, not just for built-in functions
+        # for now let's be consistent with ArgErrUnknownKwds
+        msg = ("got an unexpected keyword argument '%s'" % self.posonly_kwd)
+        return msg
