@@ -111,8 +111,8 @@ subscr_operations = misc.dict_to_switch({
 
 class __extend__(ast.GeneratorExp):
 
-    def build_container(self, codegen):
-        pass
+    def build_container_and_load_iter(self, codegen):
+        codegen.comprehension_load_iter()
 
     def get_generators(self):
         return self.generators
@@ -125,12 +125,19 @@ class __extend__(ast.GeneratorExp):
 
 class __extend__(ast.ListComp):
 
-    def build_container(self, codegen):
-        # XXX: this is suboptimal: if we use BUILD_LIST_FROM_ARG it's faster
-        # because it preallocates the list; however, we cannot use it because
-        # at this point we only have the iterator, not the original iterable
-        # object
-        codegen.emit_op_arg(ops.BUILD_LIST, 0)
+    def build_container_and_load_iter(self, codegen):
+        single = False
+        if len(self.generators) == 1:
+            gen, = self.generators
+            assert isinstance(gen, ast.comprehension)
+            if not gen.ifs:
+                single = True
+        if single:
+            codegen.comprehension_load_iter()
+            codegen.emit_op(ops.BUILD_LIST_FROM_ARG)
+        else:
+            codegen.emit_op_arg(ops.BUILD_LIST, 0)
+            codegen.comprehension_load_iter()
 
     def get_generators(self):
         return self.generators
@@ -142,8 +149,9 @@ class __extend__(ast.ListComp):
 
 class __extend__(ast.SetComp):
 
-    def build_container(self, codegen):
+    def build_container_and_load_iter(self, codegen):
         codegen.emit_op_arg(ops.BUILD_SET, 0)
+        codegen.comprehension_load_iter()
 
     def get_generators(self):
         return self.generators
@@ -155,8 +163,9 @@ class __extend__(ast.SetComp):
 
 class __extend__(ast.DictComp):
 
-    def build_container(self, codegen):
+    def build_container_and_load_iter(self, codegen):
         codegen.emit_op_arg(ops.BUILD_MAP, 0)
+        codegen.comprehension_load_iter()
 
     def get_generators(self):
         return self.generators
@@ -200,16 +209,6 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         """Override in subclasses to compile a scope."""
         raise NotImplementedError
 
-    def current_temporary_name(self):
-        """Return the name of the current temporary variable.
-
-        This must be in sync with the one during symbol table building.
-        """
-        name = "_[%d]" % (self.temporary_name_counter,)
-        self.temporary_name_counter += 1
-        assert self.scope.lookup(name) != symtable.SCOPE_UNKNOWN
-        return name
-
     def sub_scope(self, kind, name, node, lineno):
         """Convenience function for compiling a sub scope."""
         if self.scope.lookup(name) == symtable.SCOPE_GLOBAL_EXPLICIT:
@@ -305,7 +304,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
             return False
 
     def _maybe_setup_annotations(self):
-        # if the scope contained an annotated variable assignemt,
+        # if the scope contained an annotated variable assignment,
         # this will emit the requisite SETUP_ANNOTATIONS
         if self.scope.contains_annotated and not isinstance(self, AbstractFunctionCodeGenerator):
             return self.emit_op(ops.SETUP_ANNOTATIONS)
@@ -393,6 +392,16 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
             self.emit_op_arg(ops.BUILD_CONST_KEY_MAP, l)
         return l
 
+    def _visit_defaults(self, defaults):
+        assert len(defaults) > 0
+        w_tup = self._tuple_of_consts(defaults)
+        if w_tup:
+            self.update_position(defaults[-1].lineno, True)
+            self.load_const(w_tup)
+        else:
+            self.visit_sequence(defaults)
+            self.emit_op_arg(ops.BUILD_TUPLE, len(defaults))
+
     @specialize.arg(2)
     def _visit_function(self, func, function_code_generator):
         self.update_position(func.lineno, True)
@@ -403,11 +412,10 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         assert isinstance(args, ast.arguments)
 
         oparg = 0
-        self.visit_sequence(args.defaults)
 
         if args.defaults is not None and len(args.defaults):
             oparg = oparg | 0x01
-            self.emit_op_arg(ops.BUILD_TUPLE, len(args.defaults))
+            self._visit_defaults(args.defaults)
 
         if args.kwonlyargs:
             kw_default_count = self._visit_kwonlydefaults(args)
@@ -438,12 +446,10 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         args = lam.args
         assert isinstance(args, ast.arguments)
 
-        self.visit_sequence(args.defaults)
-
         oparg = 0
         if args.defaults is not None and len(args.defaults):
             oparg = oparg | 0x01
-            self.emit_op_arg(ops.BUILD_TUPLE, len(args.defaults))
+            self._visit_defaults(args.defaults)
 
         if args.kwonlyargs:
             kw_default_count = self._visit_kwonlydefaults(args)
@@ -507,10 +513,9 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
     def visit_Assert(self, asrt):
         if self.compile_info.optimize >= 1:
             return
+        assert self.compile_info.optimize == 0
         self.update_position(asrt.lineno)
         end = self.new_block()
-        if self.compile_info.optimize != 0:
-            self.emit_jump(ops.JUMP_IF_NOT_DEBUG, end)
         asrt.test.accept_jump_if(self, True, end)
         self.emit_op_name(ops.LOAD_GLOBAL, self.names, "AssertionError")
         if asrt.msg:
@@ -543,7 +548,8 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
     def visit_If(self, if_):
         self.update_position(if_.lineno, True)
         end = self.new_block()
-        test_constant = if_.test.as_constant_truth(self.space)
+        test_constant = if_.test.as_constant_truth(
+            self.space, self.compile_info)
         if test_constant == optimize.CONST_FALSE:
             self.visit_sequence(if_.orelse)
         elif test_constant == optimize.CONST_TRUE:
@@ -616,6 +622,8 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         self.use_next_block(end)
 
     def visit_AsyncFor(self, fr):
+        if not self._check_async_function():
+            self.error("'async for' outside async function", fr)
         self.update_position(fr.lineno, True)
         b_try = self.new_block()
         b_except = self.new_block()
@@ -687,7 +695,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
 
     def visit_While(self, wh):
         self.update_position(wh.lineno, True)
-        test_constant = wh.test.as_constant_truth(self.space)
+        test_constant = wh.test.as_constant_truth(self.space, self.compile_info)
         if test_constant == optimize.CONST_FALSE:
             self.visit_sequence(wh.orelse)
         else:
@@ -1003,11 +1011,12 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
                 attr = target.value
                 self._annotation_evaluate(attr)
         elif isinstance(target, ast.Subscript):
-            # similar to the above, `a[0:5]: int` evaluates the name and the slice argument
-            # and if not in a function, also evaluates the annotation
-            sl = target.slice
-            self._annotation_evaluate(target.value)
-            self._annotation_eval_slice(sl)
+            if not assign.value:
+                # similar to the above, `a[0:5]: int` evaluates the name and the slice argument
+                # and if not in a function, also evaluates the annotation
+                sl = target.slice
+                self._annotation_evaluate(target.value)
+                self._annotation_eval_slice(sl)
         else:
             self.error("can't handle annotation with %s" % (target,), target)
         # if this is not in a function, evaluate the annotation
@@ -1060,6 +1069,8 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         self.pop_frame_block(F_BLOCK_FINALLY_END, cleanup)
 
     def visit_AsyncWith(self, wih):
+        if not self._check_async_function():
+            self.error("'async with' outside async function", wih)
         self.update_position(wih.lineno, True)
         self.handle_withitem(wih, 0, is_async=True)
 
@@ -1208,7 +1219,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         count = len(elts) if elts is not None else 0
         consts_w = [None] * count
         for i in range(count):
-            w_value = elts[i].as_constant()
+            w_value = elts[i].as_constant(self.space, self.compile_info)
             if w_value is None:
                 # Not all constants
                 return None
@@ -1343,11 +1354,16 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
             if len(d.keys) < 0xffff:
                 all_constant_keys_w = []
                 for key in d.keys:
-                    if key is None or key.as_constant() is None:
+                    if key is None:
+                        constant_key = None
+                    else:
+                        constant_key = key.as_constant(
+                            self.space, self.compile_info)
+                    if constant_key is None:
                         all_constant_keys_w = None
                         break
                     else:
-                        all_constant_keys_w.append(key.as_constant())
+                        all_constant_keys_w.append(constant_key)
             for i in range(len(d.values)):
                 key = d.keys[i]
                 is_unpacking = key is None
@@ -1399,99 +1415,20 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
             self.load_const(self.space.newtext(keyword.arg))
         keyword.value.walkabout(self)
 
-    def _make_call(self, n, # args already pushed
-                   args, keywords):
-        call_type = 0
-        # the number of tuples and dictionaries on the stack
-        nsubargs = 0
-        nsubkwargs = 0
-        nkw = 0
-        nseen = 0 # the number of positional arguments on the stack
-        if args is not None:
-            for elt in args:
-                if isinstance(elt, ast.Starred):
-                    # A star-arg. If we've seen positional arguments,
-                    # pack the positional arguments into a tuple.
-                    if nseen:
-                        self.emit_op_arg(ops.BUILD_TUPLE, nseen)
-                        nseen = 0
-                        nsubargs += 1
-                    elt.value.walkabout(self)
-                    nsubargs += 1
-                elif nsubargs:
-                    # We've seen star-args already, so we
-                    # count towards items-to-pack-into-tuple.
-                    elt.walkabout(self)
-                    nseen += 1
-                else:
-                    # Positional arguments before star-arguments
-                    # are left on the stack.
-                    elt.walkabout(self)
-                    n += 1
-            if nseen:
-                # Pack up any trailing positional arguments.
-                self.emit_op_arg(ops.BUILD_TUPLE, nseen)
-                nsubargs += 1
-            if nsubargs:
-                call_type |= 1
-                if nsubargs > 1:
-                    # If we ended up with more than one stararg, we need
-                    # to concatenate them into a single sequence.
-                    self.emit_op_arg(ops.BUILD_LIST_UNPACK, nsubargs)
+    def _load_constant_tuple(self, content_w):
+        self.load_const(self.space.newtuple(content_w[:]))
 
-        # Repeat procedure for keyword args
-        nseen = 0 # the number of keyword arguments on the stack following
-        if keywords is not None:
-            for kw in keywords:
-                assert isinstance(kw, ast.keyword)
-                if kw.arg is None:
-                    # A keyword argument unpacking.
-                    if nseen:
-                        self.emit_op_arg(ops.BUILD_MAP, nseen)
-                        nseen = 0
-                        nsubkwargs += 1
-                    kw.value.walkabout(self)
-                    nsubkwargs += 1
-                elif nsubkwargs:
-                    # A keyword argument and we already have a dict.
-                    self.load_const(self.space.newtext(kw.arg))
-                    kw.value.walkabout(self)
-                    nseen += 1
-                else:
-                    # keyword argument
-                    kw.walkabout(self)
-                    nkw += 1
-            if nseen:
-                # Pack up any trailing keyword arguments.
-                self.emit_op_arg(ops.BUILD_MAP,nseen)
-                nsubkwargs += 1
-            if nsubkwargs:
-                call_type |= 2
-                if nsubkwargs > 1:
-                    # Pack it all up
-                    function_pos = n + (call_type & 1) + nkw + 1
-                    self.emit_op_arg(ops.BUILD_MAP_UNPACK_WITH_CALL, (nsubkwargs | (function_pos << 8)))
-
-        assert n < 1<<8
-        assert nkw < 1<<24
-        n |= nkw << 8;
-
-        op = 0
-        if call_type == 0:
-            op = ops.CALL_FUNCTION
-        elif call_type == 1:
-            op = ops.CALL_FUNCTION_VAR
-        elif call_type == 2:
-            op = ops.CALL_FUNCTION_KW
-        elif call_type == 3:
-            op = ops.CALL_FUNCTION_VAR_KW
-        self.emit_op_arg(op, n)
+    def _make_call(self, nargs_pushed, args, keywords):
+        space = self.space
+        CallCodeGenerator(self, nargs_pushed, args, keywords).emit_call()
 
     def visit_Call(self, call):
         self.update_position(call.lineno)
         if self._optimize_method_call(call):
             return
         call.func.walkabout(self)
+        #if getattr(call.func, "id", None) == "f":
+        #    import pdb; pdb.set_trace()
         self._make_call(0, call.args, call.keywords)
 
     def _call_has_no_star_args(self, call):
@@ -1510,6 +1447,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         return self._call_has_no_star_args(call) and not call.keywords
 
     def _optimize_method_call(self, call):
+        space = self.space
         if not self._call_has_no_star_args(call) or \
            not isinstance(call.func, ast.Attribute):
             return False
@@ -1519,9 +1457,18 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         self.emit_op_name(ops.LOOKUP_METHOD, self.names, attr_lookup.attr)
         self.visit_sequence(call.args)
         arg_count = len(call.args) if call.args is not None else 0
-        self.visit_sequence(call.keywords)
-        kwarg_count = len(call.keywords) if call.keywords is not None else 0
-        self.emit_op_arg(ops.CALL_METHOD, (kwarg_count << 8) | arg_count)
+        if not call.keywords:
+            self.emit_op_arg(ops.CALL_METHOD, arg_count)
+        else:
+            keyword_names_w = []
+            for kw in call.keywords:
+                assert isinstance(kw, ast.keyword)
+                assert kw.arg  # checked by self._call_has_no_star_args
+                w_name = space.newtext(kw.arg)
+                keyword_names_w.append(misc.intern_if_common_string(space, w_name))
+                kw.value.walkabout(self)
+            self._load_constant_tuple(keyword_names_w)
+            self.emit_op_arg(ops.CALL_METHOD_KW, len(keyword_names_w) + arg_count)
         return True
 
     def visit_ListComp(self, lc):
@@ -1542,10 +1489,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         anchor = self.new_block()
         gen = generators[gen_index]
         assert isinstance(gen, ast.comprehension)
-        if gen_index == 0:
-            self.argcount = 1
-            self.emit_op_arg(ops.LOAD_FAST, 0)
-        else:
+        if gen_index > 0:
             gen.iter.walkabout(self)
             self.emit_op(ops.GET_ITER)
         self.use_next_block(start)
@@ -1575,10 +1519,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         b_anchor = self.new_block()
         gen = generators[gen_index]
         assert isinstance(gen, ast.comprehension)
-        if gen_index == 0:
-            self.argcount = 1
-            self.emit_op_arg(ops.LOAD_FAST, 0)
-        else:
+        if gen_index > 0:
             gen.iter.walkabout(self)
             self.emit_op(ops.GET_AITER)
             self.load_const(self.space.w_None)
@@ -1630,11 +1571,11 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
     def _compile_comprehension(self, node, name, sub_scope):
         is_async_function = self.scope.is_coroutine
         code, qualname = self.sub_scope(sub_scope, name, node, node.lineno)
-        is_async_generator = self.symbols.find_scope(node).is_coroutine
-
-        if is_async_generator and not is_async_function:
-            self.error("asynchronous comprehension outside of "
-                       "an asynchronous function", node)
+        is_async_comprehension = self.symbols.find_scope(node).is_coroutine
+        if is_async_comprehension and not is_async_function:
+            if not isinstance(node, ast.GeneratorExp):
+                self.error("asynchronous comprehension outside of "
+                           "an asynchronous function", node)
 
         self.update_position(node.lineno)
         self._make_function(code, qualname=qualname)
@@ -1648,7 +1589,7 @@ class PythonCodeGenerator(assemble.PythonCodeMaker):
         else:
             self.emit_op(ops.GET_ITER)
         self.emit_op_arg(ops.CALL_FUNCTION, 1)
-        if is_async_generator and sub_scope is not GenExpCodeGenerator:
+        if is_async_comprehension and sub_scope is not GenExpCodeGenerator:
             self.emit_op(ops.GET_AWAITABLE)
             self.load_const(self.space.w_None)
             self.emit_op(ops.YIELD_FROM)
@@ -1875,12 +1816,16 @@ class LambdaCodeGenerator(AbstractFunctionCodeGenerator):
 class ComprehensionCodeGenerator(AbstractFunctionCodeGenerator):
 
     def _compile(self, node):
+        self.argcount = 1
         assert isinstance(node, ast.expr)
         self.update_position(node.lineno)
-        node.build_container(self)
+        node.build_container_and_load_iter(self)
         self._comp_generator(node, node.get_generators(), 0)
         self._end_comp()
 
+    def comprehension_load_iter(self):
+        self.emit_op_arg(ops.LOAD_FAST, 0)
+
     def _end_comp(self):
         self.emit_op(ops.RETURN_VALUE)
 
@@ -1933,3 +1878,127 @@ class ClassCodeGenerator(PythonCodeGenerator):
         if self.scope.doc_removable:
             flags |= consts.CO_KILL_DOCSTRING
         return PythonCodeGenerator._get_code_flags(self) | flags
+
+
+class CallCodeGenerator(object):
+    def __init__(self, codegenerator, nargs_pushed, args, keywords):
+        self.space = codegenerator.space
+        self.codegenerator = codegenerator
+        self.nargs_pushed = nargs_pushed
+        self.args = args
+        self.keywords = keywords
+
+        # the number of tuples and dictionaries on the stack
+        self.nsubargs = 0
+        self.nsubkwargs = 0
+        self.keyword_names_w = []
+
+    def _pack_positional_into_tuple(self):
+        if self.nargs_pushed:
+            self.codegenerator.emit_op_arg(ops.BUILD_TUPLE, self.nargs_pushed)
+            self.nsubargs += 1
+            self.nargs_pushed = 0
+
+    def _push_args(self):
+        for elt in self.args:
+            if isinstance(elt, ast.Starred):
+                # we have a *arg
+                self._pack_positional_into_tuple()
+                elt.value.walkabout(self.codegenerator)
+                self.nsubargs += 1
+                continue
+            if self.nargs_pushed >= MAX_STACKDEPTH_CONTAINERS // 2:
+                # stack depth getting too big
+                self._pack_positional_into_tuple()
+            elt.walkabout(self.codegenerator)
+            self.nargs_pushed += 1
+        if self.nsubargs:
+            # Pack up any trailing positional arguments.
+            self._pack_positional_into_tuple()
+            if self.nsubargs > 1:
+                # If we ended up with more than one stararg, we need
+                # to concatenate them into a single sequence.
+                # XXX CPython uses BUILD_TUPLE_UNPACK_WITH_CALL, but I
+                # don't quite see the difference?
+                self.codegenerator.emit_op_arg(ops.BUILD_TUPLE_UNPACK, self.nsubargs)
+
+    def _pack_kwargs_into_dict(self):
+        if self.keyword_names_w:
+            self.codegenerator._load_constant_tuple(self.keyword_names_w)
+            # XXX use BUILD_MAP for size 1?
+            self.codegenerator.emit_op_arg(ops.BUILD_CONST_KEY_MAP, len(self.keyword_names_w))
+            self.keyword_names_w = []
+            self.nsubkwargs += 1
+
+    def _push_kwargs(self):
+        for kw in self.keywords:
+            assert isinstance(kw, ast.keyword)
+            if kw.arg is None:
+                # if we see **args or if the number of keywords is huge,
+                # pack up keywords on the stack so far
+                self._pack_kwargs_into_dict()
+                kw.value.walkabout(self.codegenerator)
+                self.nsubkwargs += 1
+                continue
+            if len(self.keyword_names_w) > MAX_STACKDEPTH_CONTAINERS // 2:
+                self._pack_kwargs_into_dict()
+            w_name = self.space.newtext(kw.arg)
+            self.keyword_names_w.append(misc.intern_if_common_string(self.space, w_name))
+            kw.value.walkabout(self.codegenerator)
+        if self.nsubkwargs:
+            self._pack_kwargs_into_dict()
+            if self.nsubkwargs > 1:
+                # Pack it all up
+                self.codegenerator.emit_op_arg(ops.BUILD_MAP_UNPACK_WITH_CALL, self.nsubkwargs)
+
+    def _pack_positional_args_into_tuple(self):
+        if self.nargs_pushed == 0:
+            self.codegenerator._load_constant_tuple([])
+        else:
+            self.codegenerator.emit_op_arg(ops.BUILD_TUPLE, self.nargs_pushed)
+        self.nsubargs += 1
+
+    def _push_tuple_positional_args_if_necessary(self):
+        if self.nsubargs:
+            # can't use CALL_FUNCTION_KW anyway, because we already have a
+            # tuple as the positional args
+            return
+        # we might get away with using CALL_FUNCTION_KW if there are no **kwargs
+        for kw in self.keywords:
+            assert isinstance(kw, ast.keyword)
+            if kw.arg is None:
+                # we found a **kwarg, thus we're using CALL_FUNCTION_EX, we
+                # need to pack up positional arguments first
+                self._pack_positional_args_into_tuple()
+                break
+        if self.nsubargs == 0 and len(self.keywords) > MAX_STACKDEPTH_CONTAINERS // 2:
+            # we have a huge amount of keyword args, thus we also need to use
+            # CALL_FUNCTION_EX
+            self._pack_positional_args_into_tuple()
+
+    def emit_call(self):
+        keywords = self.keywords
+        codegenerator = self.codegenerator
+        space = self.space
+        if self.args is not None:
+            self._push_args()
+
+        # Repeat procedure for keyword args
+        if keywords is None or len(keywords) == 0:
+            if not self.nsubargs:
+                # no *args, no keyword args, no **kwargs
+                codegenerator.emit_op_arg(ops.CALL_FUNCTION, self.nargs_pushed)
+                return
+        else:
+            self._push_tuple_positional_args_if_necessary()
+            self._push_kwargs()
+
+        if self.nsubkwargs == 0 and self.nsubargs == 0:
+            # can use CALL_FUNCTION_KW
+            assert len(self.keyword_names_w) > 0 # otherwise we would have used CALL_FUNCTION
+            codegenerator._load_constant_tuple(self.keyword_names_w)
+            codegenerator.emit_op_arg(ops.CALL_FUNCTION_KW, self.nargs_pushed + len(self.keyword_names_w))
+        else:
+            self._pack_kwargs_into_dict()
+            codegenerator.emit_op_arg(ops.CALL_FUNCTION_EX, int(self.nsubkwargs > 0))
+
