@@ -48,6 +48,7 @@ class SignatureBuilder(object):
         self.argnames = argnames
         self.varargname = varargname
         self.kwargname = kwargname
+        self.posonlyargnames = None
         self.kwonlyargnames = None
 
     def append(self, argname):
@@ -56,13 +57,19 @@ class SignatureBuilder(object):
         else:
             self.kwonlyargnames.append(argname)
 
+    def marker_posonly(self):
+        assert self.posonlyargnames is None
+        assert self.kwonlyargnames is None
+        self.posonlyargnames = self.argnames
+        self.argnames = []
+
     def marker_kwonly(self):
         assert self.kwonlyargnames is None
         self.kwonlyargnames = []
 
     def signature(self):
         return Signature(self.argnames, self.varargname, self.kwargname,
-                         self.kwonlyargnames)
+                         self.kwonlyargnames, self.posonlyargnames)
 
 #________________________________________________________________
 
@@ -253,6 +260,11 @@ class UnwrapSpec_Check(UnwrapSpecRecipe):
         name = int_unwrapping_space_method(typ)
         self.checked_space_method(name, app_sig)
 
+    def visit_posonly(self, _, app_sig):
+        argname = self.orig_arg()
+        assert argname == '__posonly__'
+        app_sig.marker_posonly()
+
     def visit_kwonly(self, _, app_sig):
         argname = self.orig_arg()
         assert argname == '__kwonly__'
@@ -363,6 +375,9 @@ class UnwrapSpec_EmitRun(UnwrapSpecEmit):
     def visit_truncatedint_w(self, typ):
         self.run_args.append("space.truncatedint_w(%s)" % (self.scopenext(),))
 
+    def visit_posonly(self, typ):
+        self.run_args.append("None")
+
     def visit_kwonly(self, typ):
         self.run_args.append("None")
 
@@ -536,6 +551,9 @@ class UnwrapSpec_FastFunc_Unwrap(UnwrapSpecEmit):
     def visit_truncatedint_w(self, typ):
         self.unwrap.append("space.truncatedint_w(%s)" % (self.nextarg(),))
 
+    def visit_posonly(self, typ):
+        raise FastFuncNotSupported
+
     def visit_kwonly(self, typ):
         raise FastFuncNotSupported
 
@@ -645,6 +663,8 @@ def build_unwrap_spec(func, argnames, self_type=None):
                 unwrap_spec.append('args_w')
             elif argname.startswith('w_'):
                 unwrap_spec.append(W_Root)
+            elif argname == '__posonly__':
+                unwrap_spec.append('posonly')
             elif argname == '__kwonly__':
                 unwrap_spec.append('kwonly')
             else:
@@ -702,6 +722,8 @@ class BuiltinCode(Code):
         argnames = sig.argnames
         varargname = sig.varargname
         kwargname = sig.kwargname
+        if sig.posonlyargnames:
+            import pdb; pdb.set_trace()
         if sig.kwonlyargnames:
             import pdb; pdb.set_trace()
         self._argnames = argnames
@@ -755,6 +777,7 @@ class BuiltinCode(Code):
                     self.func__args__ = func
                 elif unwrap_spec == [self_type, ObjSpace, Arguments]:
                     self.__class__ = BuiltinCodePassThroughArguments1
+                    self.descr_reqcls = self_type
                     miniglobals = {'func': func, 'self_type': self_type}
                     d = {}
                     source = """if 1:
@@ -793,6 +816,7 @@ class BuiltinCode(Code):
     def funcrun_obj(self, func, w_obj, args):
         space = func.space
         activation = self.activation
+
         scope_w = args.parse_obj(w_obj, func.name, self.sig,
                                  func.defs_w, func.w_kw_defs, self.minargs)
         try:
@@ -800,10 +824,7 @@ class BuiltinCode(Code):
         except DescrMismatch:
             if w_obj is not None:
                 args = args.prepend(w_obj)
-            return scope_w[0].descr_call_mismatch(space,
-                                                  self.descrmismatch_op,
-                                                  self.descr_reqcls,
-                                                  args)
+            return self._type_unwrap_mismatch(space, args)
         except Exception as e:
             self.handle_exception(space, e)
             w_result = None
@@ -811,6 +832,15 @@ class BuiltinCode(Code):
             w_result = space.w_None
         return w_result
 
+    def _type_unwrap_mismatch(self, space, args):
+        w_obj = args.firstarg()
+        if w_obj is None:
+            raise oefmt(space.w_SystemError, "unexpected DescrMismatch error")
+        return w_obj.descr_call_mismatch(space,
+                                         self.descrmismatch_op,
+                                         self.descr_reqcls,
+                                         args)
+
     def handle_exception(self, space, e):
         try:
             if not we_are_translated():
@@ -833,10 +863,7 @@ class BuiltinCodePassThroughArguments0(BuiltinCode):
         try:
             w_result = self.func__args__(space, args)
         except DescrMismatch:
-            return args.firstarg().descr_call_mismatch(space,
-                                                  self.descrmismatch_op,
-                                                  self.descr_reqcls,
-                                                  args)
+            return self._type_unwrap_mismatch(space, args)
         except Exception as e:
             self.handle_exception(space, e)
             w_result = None
@@ -854,10 +881,7 @@ class BuiltinCodePassThroughArguments1(BuiltinCode):
         try:
             w_result = self.func__args__(space, w_obj, args)
         except DescrMismatch:
-            return args.firstarg().descr_call_mismatch(space,
-                                                  self.descrmismatch_op,
-                                                  self.descr_reqcls,
-                                                  args.prepend(w_obj))
+            return self._type_unwrap_mismatch(space, args.prepend(w_obj))
         except Exception as e:
             self.handle_exception(space, e)
             w_result = None
@@ -897,9 +921,7 @@ class BuiltinCode1(BuiltinCode):
         try:
             w_result = self.fastfunc_1(space, w1)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space, [w1]))
         except Exception as e:
             self.handle_exception(space, e)
@@ -923,9 +945,7 @@ class BuiltinCode2(BuiltinCode):
         try:
             w_result = self.fastfunc_2(space, w1, w2)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space, [w1, w2]))
         except Exception as e:
             self.handle_exception(space, e)
@@ -950,9 +970,7 @@ class BuiltinCode3(BuiltinCode):
         try:
             w_result = self.fastfunc_3(space, w1, w2, w3)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space, [w1, w2, w3]))
         except Exception as e:
             self.handle_exception(space, e)
@@ -978,9 +996,7 @@ class BuiltinCode4(BuiltinCode):
         try:
             w_result = self.fastfunc_4(space, w1, w2, w3, w4)
         except DescrMismatch:
-            return w1.descr_call_mismatch(space,
-                                          self.descrmismatch_op,
-                                          self.descr_reqcls,
+            return self._type_unwrap_mismatch(space,
                                           Arguments(space,
                                                     [w1, w2, w3, w4]))
         except Exception as e:
@@ -1072,7 +1088,7 @@ class interp2app(W_Root):
         alldefs_w = {}
         assert len(self._code._argnames) == len(self._code._unwrap_spec)
         for name, spec in zip(self._code._argnames, self._code._unwrap_spec):
-            if name == '__kwonly__':
+            if name in ('__posonly__', '__kwonly__'):
                 continue
 
             defaultval = self._staticdefs.get(name, NO_DEFAULT)
@@ -1114,10 +1130,12 @@ class interp2app(W_Root):
         #
         sig = self._code.sig
         first_defined = 0
-        while (first_defined < len(sig.argnames) and
-               sig.argnames[first_defined] not in alldefs_w):
+        allposargnames = sig.posonlyargnames + sig.argnames
+        n_allposargnames = len(allposargnames)
+        while (first_defined < n_allposargnames and
+               allposargnames[first_defined] not in alldefs_w):
             first_defined += 1
-        defs_w = [alldefs_w.pop(name) for name in sig.argnames[first_defined:]]
+        defs_w = [alldefs_w.pop(name) for name in allposargnames[first_defined:]]
 
         kw_defs_w = None
         if alldefs_w:
