@@ -36,6 +36,8 @@ class FrameDebugData(object):
     f_lineno                 = 0      # current lineno for tracing
     is_being_profiled        = False
     is_in_line_tracing       = False
+    f_trace_lines            = True
+    f_trace_opcodes          = False
     w_locals                 = None
     hidden_operationerr      = None
 
@@ -68,6 +70,7 @@ class PyFrame(W_Root):
     frame_finished_execution = False
     f_generator_wref         = rweakref.dead_ref  # for generators/coroutines
     f_generator_nowref       = None               # (only one of the two attrs)
+    w_yielding_from = None
     last_instr               = -1
     f_backref                = jit.vref_None
 
@@ -147,6 +150,18 @@ class PyFrame(W_Root):
             return None
         return d.w_locals
 
+    def get_f_trace_lines(self):
+        d = self.getdebug()
+        if d is None:
+            return True
+        return d.f_trace_lines
+
+    def get_f_trace_opcodes(self):
+        d = self.getdebug()
+        if d is None:
+            return False
+        return d.f_trace_opcodes
+
     @not_rpython
     def __repr__(self):
         # useful in tracebacks
@@ -238,7 +253,8 @@ class PyFrame(W_Root):
                                  "an unexpected number of free variables")
         index = code.co_nlocals
         for i in range(ncellvars):
-            self.locals_cells_stack_w[index] = Cell()
+            self.locals_cells_stack_w[index] = Cell(
+                    None, self.pycode.cell_families[i])
             index += 1
         for i in range(nfreevars):
             self.locals_cells_stack_w[index] = outer_func.closure[i]
@@ -254,29 +270,7 @@ class PyFrame(W_Root):
         if self._is_generator_or_coroutine():
             return self.initialize_as_generator(name, qualname)
         else:
-            # untranslated: check that sys_exc_info is exactly
-            # restored after running any Python function.
-            # Translated: actually save and restore it, as an attempt to
-            # work around rare cases that can occur if RecursionError or
-            # MemoryError is raised at just the wrong place
-            executioncontext = self.space.getexecutioncontext()
-            exc_on_enter = executioncontext.sys_exc_info()
-            if we_are_translated():
-                try:
-                    return self.execute_frame()
-                finally:
-                    executioncontext.set_sys_exc_info(exc_on_enter)
-            else:
-                # untranslated, we check consistency, but not in case of
-                # interp-level exceptions different than OperationError
-                # (e.g. a random failing test, or a pytest Skipped exc.)
-                try:
-                    w_res = self.execute_frame()
-                    assert exc_on_enter is executioncontext.sys_exc_info()
-                except OperationError:
-                    assert exc_on_enter is executioncontext.sys_exc_info()
-                    raise
-                return w_res
+            return self.execute_frame()
     run._always_inline_ = True
 
     def initialize_as_generator(self, name, qualname):
@@ -287,6 +281,7 @@ class PyFrame(W_Root):
             gen = Coroutine(self, name, qualname)
             ec = space.getexecutioncontext()
             w_wrapper = ec.w_coroutine_wrapper_fn
+            gen.capture_origin(ec)
         elif flags & pycode.CO_ASYNC_GENERATOR:
             from pypy.interpreter.generator import AsyncGenerator
             gen = AsyncGenerator(self, name, qualname)
@@ -319,9 +314,40 @@ class PyFrame(W_Root):
                 ec.in_coroutine_wrapper = False
         return w_gen
 
-    def execute_frame(self, in_generator=None, w_arg_or_err=None):
+    def resume_execute_frame(self, w_arg_or_err):
+        # Called from execute_frame() just before resuming the bytecode
+        # interpretation.
+        from pypy.interpreter.pyopcode import SApplicationException
+        space = self.space
+        w_yf = self.w_yielding_from
+        if w_yf is not None:
+            self.w_yielding_from = None
+            try:
+                self.next_yield_from(w_yf, w_arg_or_err)
+            except OperationError as operr:
+                operr.record_context(space, space.getexecutioncontext())
+                return self.handle_generator_error(operr)
+            # Normal case: the call above raises Yield.
+            # We reach this point if the iterable is exhausted.
+            last_instr = jit.promote(self.last_instr)
+            assert last_instr & 1 == 0
+            assert last_instr >= 0
+            return r_uint(last_instr + 2)
+
+        if isinstance(w_arg_or_err, SApplicationException):
+            return self.handle_generator_error(w_arg_or_err.operr)
+
+        last_instr = jit.promote(self.last_instr)
+        if last_instr != -1:
+            assert last_instr & 1 == 0
+            self.pushvalue(w_arg_or_err)
+            return r_uint(last_instr + 2)
+        else:
+            return r_uint(0)
+
+    def execute_frame(self, w_arg_or_err=None):
         """Execute this frame.  Main entry point to the interpreter.
-        'in_generator' is non-None iff we are starting or resuming
+        'w_arg_or_err' is non-None iff we are starting or resuming
         a generator or coroutine frame; in that case, w_arg_or_err
         is the input argument -or- an SApplicationException instance.
         """
@@ -343,12 +369,11 @@ class PyFrame(W_Root):
             # the YIELD_VALUE/YIELD_FROM instruction.
             try:
                 try:
-                    if in_generator is None:
+                    if w_arg_or_err is None:
                         assert self.last_instr == -1
                         next_instr = r_uint(0)
                     else:
-                        next_instr = in_generator.resume_execute_frame(
-                                                        self, w_arg_or_err)
+                        next_instr = self.resume_execute_frame(w_arg_or_err)
                 except pyopcode.Yield:
                     w_exitvalue = self.popvalue()
                 else:
@@ -378,9 +403,12 @@ class PyFrame(W_Root):
         assert self.locals_cells_stack_w[depth] is None
         self.valuestackdepth = depth + 1
 
+    def assert_stack_index(self, index):
+        if we_are_translated():
+            return
+        assert self._check_stack_index(index)
+
     def _check_stack_index(self, index):
-        # will be completely removed by the optimizer if only used in an assert
-        # and if asserts are disabled
         code = self.pycode
         ncellvars = len(code.co_cellvars)
         nfreevars = len(code.co_freevars)
@@ -392,7 +420,7 @@ class PyFrame(W_Root):
 
     def popvalue_maybe_none(self):
         depth = self.valuestackdepth - 1
-        assert self._check_stack_index(depth)
+        self.assert_stack_index(depth)
         assert depth >= 0
         w_object = self.locals_cells_stack_w[depth]
         self.locals_cells_stack_w[depth] = None
@@ -421,7 +449,7 @@ class PyFrame(W_Root):
     def peekvalues(self, n):
         values_w = [None] * n
         base = self.valuestackdepth - n
-        assert self._check_stack_index(base)
+        self.assert_stack_index(base)
         assert base >= 0
         while True:
             n -= 1
@@ -434,7 +462,7 @@ class PyFrame(W_Root):
     def dropvalues(self, n):
         n = hint(n, promote=True)
         finaldepth = self.valuestackdepth - n
-        assert self._check_stack_index(finaldepth)
+        self.assert_stack_index(finaldepth)
         assert finaldepth >= 0
         while True:
             n -= 1
@@ -470,14 +498,14 @@ class PyFrame(W_Root):
     def peekvalue_maybe_none(self, index_from_top=0):
         index_from_top = hint(index_from_top, promote=True)
         index = self.valuestackdepth + ~index_from_top
-        assert self._check_stack_index(index)
+        self.assert_stack_index(index)
         assert index >= 0
         return self.locals_cells_stack_w[index]
 
     def settopvalue(self, w_object, index_from_top=0):
         index_from_top = hint(index_from_top, promote=True)
         index = self.valuestackdepth + ~index_from_top
-        assert self._check_stack_index(index)
+        self.assert_stack_index(index)
         assert index >= 0
         self.locals_cells_stack_w[index] = ll_assert_not_none(w_object)
 
@@ -495,6 +523,7 @@ class PyFrame(W_Root):
         """ Returns 'funcname()' from either a function name fnname or a
         wrapped callable w_function. If it's not a function or a method, returns
         'Classname object'"""
+        # XXX this is super annoying to compute every time we do a function call!
         # CPython has a similar function, PyEval_GetFuncName
         from pypy.interpreter.function import Function, Method
         if fnname is not None:
@@ -505,7 +534,7 @@ class PyFrame(W_Root):
             return w_function.name + '()'
         if isinstance(w_function, Method):
             return self._guess_function_name_parens(None, w_function.w_function)
-        return w_function.getname(self.space) + ' object'
+        return self.space.type(w_function).getname(self.space) + ' object'
 
     def make_arguments(self, nargs, methodcall=False, w_function=None, fnname=None):
         fnname_parens = self._guess_function_name_parens(fnname, w_function)
@@ -556,7 +585,7 @@ class PyFrame(W_Root):
         # Copy values from the fastlocals to self.w_locals
         d = self.getorcreatedebug()
         if d.w_locals is None:
-            d.w_locals = self.space.newdict()
+            d.w_locals = self.space.newdict(module=True)
         varnames = self.getcode().getvarnames()
         for i in range(min(len(varnames), self.getcode().co_nlocals)):
             name = varnames[i]
@@ -887,10 +916,17 @@ class PyFrame(W_Root):
     def fdel_f_trace(self, space):
         self.getorcreatedebug().w_f_trace = None
 
-    def fget_f_restricted(self, space):
-        if space.config.objspace.honor__builtins__:
-            return space.newbool(self.builtin is not space.builtin)
-        return space.w_False
+    def fget_f_trace_lines(self, space):
+        return space.newbool(self.get_f_trace_lines())
+
+    def fset_f_trace_lines(self, space, w_trace):
+        self.getorcreatedebug().f_trace_lines = space.is_true(w_trace)
+
+    def fget_f_trace_opcodes(self, space):
+        return space.newbool(self.get_f_trace_opcodes())
+
+    def fset_f_trace_opcodes(self, space, w_trace):
+        self.getorcreatedebug().f_trace_opcodes = space.is_true(w_trace)
 
     def get_generator(self):
         if self.space.config.translation.rweakref:
@@ -927,7 +963,8 @@ class PyFrame(W_Root):
         for i in range(len(self.locals_cells_stack_w)):
             w_oldvalue = self.locals_cells_stack_w[i]
             if isinstance(w_oldvalue, Cell):
-                w_newvalue = Cell()
+                w_newvalue = Cell(
+                    None, w_oldvalue.family)
             else:
                 w_newvalue = None
             self.locals_cells_stack_w[i] = w_newvalue
@@ -942,6 +979,12 @@ class PyFrame(W_Root):
             self.space, operr, self, self.last_instr)
         raise operr
 
+    def descr_repr(self, space):
+        code = self.pycode
+        moreinfo = ", file '%s', line %s, code %s" % (
+            code.co_filename, self.get_last_lineno(), code.co_name)
+        return self.getrepr(space, "frame", moreinfo)
+
 # ____________________________________________________________
 
 def get_block_class(opname):
