@@ -178,10 +178,13 @@ class __extend__(pyframe.PyFrame):
                 oparg = (oparg * 256) | arg
 
             if opcode == opcodedesc.RETURN_VALUE.index:
+                if not self.blockstack_non_empty():
+                    self.frame_finished_execution = True  # for generators
+                    raise Return
                 w_returnvalue = self.popvalue()
                 block = self.unrollstack(SReturnValue.kind)
                 if block is None:
-                    self.pushvalue(w_returnvalue)   # XXX ping pong
+                    self.pushvalue(w_returnvalue)
                     raise Return
                 else:
                     unroller = SReturnValue(w_returnvalue)
@@ -273,12 +276,12 @@ class __extend__(pyframe.PyFrame):
                 self.CALL_FUNCTION(oparg, next_instr)
             elif opcode == opcodedesc.CALL_FUNCTION_KW.index:
                 self.CALL_FUNCTION_KW(oparg, next_instr)
-            elif opcode == opcodedesc.CALL_FUNCTION_VAR.index:
-                self.CALL_FUNCTION_VAR(oparg, next_instr)
-            elif opcode == opcodedesc.CALL_FUNCTION_VAR_KW.index:
-                self.CALL_FUNCTION_VAR_KW(oparg, next_instr)
+            elif opcode == opcodedesc.CALL_FUNCTION_EX.index:
+                self.CALL_FUNCTION_EX(oparg, next_instr)
             elif opcode == opcodedesc.CALL_METHOD.index:
                 self.CALL_METHOD(oparg, next_instr)
+            elif opcode == opcodedesc.CALL_METHOD_KW.index:
+                self.CALL_METHOD_KW(oparg, next_instr)
             elif opcode == opcodedesc.COMPARE_OP.index:
                 self.COMPARE_OP(oparg, next_instr)
             elif opcode == opcodedesc.DELETE_ATTR.index:
@@ -753,13 +756,7 @@ class __extend__(pyframe.PyFrame):
         space.call_method(w_globals, 'setdefault', space.newtext('__builtins__'),
                           self.get_builtin())
 
-        plain = (self.get_w_locals() is not None and
-                 space.is_w(w_locals, self.get_w_locals()))
-        if plain:
-            w_locals = self.getdictscope()
         code.exec_code(space, w_globals, w_locals)
-        if plain:
-            self.setdictscope(w_locals)
 
     def POP_EXCEPT(self, oparg, next_instr):
         block = self.pop_block()
@@ -1113,24 +1110,51 @@ class __extend__(pyframe.PyFrame):
             self.pushvalue(w_value)
         raise Yield
 
+    def next_yield_from(self, w_yf, w_inputvalue_or_err):
+        """Fetch the next item of the current 'yield from', push it on
+        the frame stack, and raises Yield.  If there isn't one, push
+        w_stopiteration_value and returns.  May also just raise.
+        """
+        from pypy.interpreter.generator import (
+            GeneratorOrCoroutine, AsyncGenASend)
+        space = self.space
+        try:
+            if isinstance(w_yf, GeneratorOrCoroutine):
+                w_retval = w_yf.send_ex(w_inputvalue_or_err)
+            elif isinstance(w_yf, AsyncGenASend):   # performance only
+                w_retval = w_yf.do_send(w_inputvalue_or_err)
+            elif space.is_w(w_inputvalue_or_err, space.w_None):
+                w_retval = space.next(w_yf)
+            else:
+                w_retval = delegate_to_nongen(space, w_yf, w_inputvalue_or_err)
+        except OperationError as e:
+            if not e.match(space, space.w_StopIteration):
+                raise
+            self._report_stopiteration_sometimes(w_yf, e)
+            try:
+                w_stop_value = space.getattr(e.get_w_value(space),
+                                             space.newtext("value"))
+            except OperationError as e:
+                if not e.match(space, space.w_AttributeError):
+                    raise
+                w_stop_value = space.w_None
+            self.pushvalue(w_stop_value)
+            return
+        else:
+            self.pushvalue(w_retval)
+            self.w_yielding_from = w_yf
+            raise Yield
+
     def YIELD_FROM(self, oparg, next_instr):
         # Unlike CPython, we handle this not by repeating the same
         # bytecode over and over until the inner iterator is exhausted.
-        # Instead, we directly set the generator's w_yielded_from.
-        # This asks generator.resume_execute_frame() to exhaust that
+        # Instead, we set w_yielding_from.
+        # This asks resume_execute_frame() to exhaust that
         # sub-iterable first before continuing on the next bytecode.
-        in_generator = self.get_generator()
-        if in_generator is None:
-            # Issue #2768: rare case involving __del__ methods.
-            # XXX This is a workaround, not a proper solution.
-            raise oefmt(self.space.w_RuntimeError,
-                        "PyPy limitation: cannot use 'yield from' or 'await' "
-                        "in a generator/coroutine that has been partially "
-                        "deallocated already, typically seen via __del__")
         w_inputvalue = self.popvalue()    # that's always w_None, actually
         w_gen = self.popvalue()
         #
-        in_generator.next_yield_from(self, w_gen, w_inputvalue)
+        self.next_yield_from(w_gen, w_inputvalue)
         # Common case: the call above raises Yield.
         # If instead the iterable is empty, next_yield_from() pushed the
         # final result and returns.  In that case, we can just continue
@@ -1318,31 +1342,54 @@ class __extend__(pyframe.PyFrame):
         self.pushvalue(w_result)
 
     def CALL_FUNCTION(self, oparg, next_instr):
-        # XXX start of hack for performance
-        if (oparg >> 8) & 0xff == 0:
-            # Only positional arguments
-            nargs = oparg & 0xff
-            w_function = self.peekvalue(nargs)
-            try:
-                w_result = self.space.call_valuestack(w_function, nargs, self)
-            finally:
-                self.dropvalues(nargs + 1)
-            self.pushvalue(w_result)
-        # XXX end of hack for performance
+        # Only positional arguments
+        nargs = oparg & 0xff
+        w_function = self.peekvalue(nargs)
+        try:
+            w_result = self.space.call_valuestack(w_function, nargs, self)
+        finally:
+            self.dropvalues(nargs + 1)
+        self.pushvalue(w_result)
+
+    @jit.unroll_safe
+    def CALL_FUNCTION_KW(self, n_arguments, next_instr):
+        w_tup_varnames = self.popvalue()
+        keywords_w = self.space.fixedview(w_tup_varnames)
+        n_keywords = len(keywords_w)
+        n_arguments -= n_keywords
+        keywords = [self.space.text_w(w_keyword) for w_keyword in keywords_w]
+        keywords_w = [None] * n_keywords
+        while True:
+            n_keywords -= 1
+            if n_keywords < 0:
+                break
+            w_value = self.popvalue()
+            keywords_w[n_keywords] = w_value
+        arguments = self.popvalues(n_arguments)
+        w_function  = self.popvalue()
+        args = self.argument_factory(arguments, keywords, keywords_w, None, None,
+                                     w_function=w_function)
+        if self.get_is_being_profiled() and function.is_builtin_code(w_function):
+            w_result = self.space.call_args_and_c_profile(self, w_function,
+                                                          args)
         else:
-            # general case
-            self.call_function(oparg)
+            w_result = self.space.call_args(w_function, args)
+        self.pushvalue(w_result)
 
-    def CALL_FUNCTION_VAR(self, oparg, next_instr):
-        self.call_function(oparg, has_vararg=True)
-
-    def CALL_FUNCTION_KW(self, oparg, next_instr):
-        w_varkw = self.popvalue()
-        self.call_function(oparg, w_varkw)
-
-    def CALL_FUNCTION_VAR_KW(self, oparg, next_instr):
-        w_varkw = self.popvalue()
-        self.call_function(oparg, w_varkw, has_vararg=True)
+    def CALL_FUNCTION_EX(self, has_kwarg, next_instr):
+        w_kwargs = None
+        if has_kwarg:
+            w_kwargs = self.popvalue()
+        w_args = self.popvalue()
+        w_function = self.popvalue()
+        args = self.argument_factory(
+            [], None, None, w_star=w_args, w_starstar=w_kwargs, w_function=w_function)
+        if self.get_is_being_profiled() and function.is_builtin_code(w_function):
+            w_result = self.space.call_args_and_c_profile(self, w_function,
+                                                          args)
+        else:
+            w_result = self.space.call_args(w_function, args)
+        self.pushvalue(w_result)
 
     @jit.unroll_safe
     def MAKE_FUNCTION(self, oparg, next_instr):
@@ -1415,6 +1462,7 @@ class __extend__(pyframe.PyFrame):
     # overridden by faster version in the standard object space.
     LOOKUP_METHOD = LOAD_ATTR
     CALL_METHOD = CALL_FUNCTION
+    CALL_METHOD_KW = CALL_FUNCTION_KW
 
     def MISSING_OPCODE(self, oparg, next_instr):
         ofs = self.last_instr
@@ -1468,30 +1516,29 @@ class __extend__(pyframe.PyFrame):
         self.pushvalue(w_set)
 
     @jit.unroll_safe
-    def list_unpack_helper(frame, itemcount):
-        space = frame.space
-        w_sum = space.newlist([], sizehint=itemcount)
-        for i in range(itemcount, 0, -1):
-            w_item = frame.peekvalue(i-1)
-            w_sum.extend(w_item)
-        frame.popvalues(itemcount)
-        return w_sum
+    def BUILD_TUPLE_UNPACK(self, itemcount, next_instr):
+        l = []
+        for i in range(itemcount-1, -1, -1):
+            w_item = self.peekvalue(i)
+            l.extend(self.space.fixedview(w_item))
+        self.popvalues(itemcount)
+        self.pushvalue(self.space.newtuple(l[:]))
 
     @jit.unroll_safe
-    def BUILD_TUPLE_UNPACK(self, itemcount, next_instr):
-        w_list = self.list_unpack_helper(itemcount)
-        items = [w_obj for w_obj in w_list.getitems_unroll()]
-        self.pushvalue(self.space.newtuple(items))
-
     def BUILD_LIST_UNPACK(self, itemcount, next_instr):
-        w_sum = self.list_unpack_helper(itemcount)
+        space = self.space
+        w_sum = space.newlist([], sizehint=itemcount)
+        for i in range(itemcount-1, -1, -1):
+            w_item = self.peekvalue(i)
+            w_sum.extend(w_item)
+        self.popvalues(itemcount)
         self.pushvalue(w_sum)
 
     def BUILD_MAP_UNPACK(self, itemcount, next_instr):
         self._build_map_unpack(itemcount, with_call=False)
 
     def BUILD_MAP_UNPACK_WITH_CALL(self, oparg, next_instr):
-        num_maps = oparg & 0xff
+        num_maps = oparg # XXX CPython generates better error messages
         self._build_map_unpack(num_maps, with_call=True)
 
     @jit.unroll_safe
@@ -1556,7 +1603,7 @@ class __extend__(pyframe.PyFrame):
         w_iterable = self.popvalue()
         w_iter = get_awaitable_iter(self.space, w_iterable)
         if isinstance(w_iter, Coroutine):
-            if w_iter.w_yielded_from is not None:
+            if w_iter.get_delegate() is not None:
                 # 'w_iter' is a coroutine object that is being awaited,
                 # '.w_yielded_from' is the current awaitable being awaited on.
                 raise oefmt(self.space.w_RuntimeError,
@@ -1694,6 +1741,27 @@ class __extend__(pyframe.PyFrame):
         else:
             self.MISSING_OPCODE(oparg, next_instr)
 
+def delegate_to_nongen(space, w_yf, w_inputvalue_or_err):
+    # invoke a "send" or "throw" by method name to a non-generator w_yf
+    if isinstance(w_inputvalue_or_err, SApplicationException):
+        operr = w_inputvalue_or_err.operr
+        try:
+            w_meth = space.getattr(w_yf, space.newtext("throw"))
+        except OperationError as e:
+            if not e.match(space, space.w_AttributeError):
+                raise
+            raise operr
+        # bah, CPython calls here with the exact same arguments as
+        # originally passed to throw().  In our case it is far removed.
+        # Let's hope nobody will complain...
+        operr.normalize_exception(space)
+        w_exc = operr.w_type
+        w_val = operr.get_w_value(space)
+        w_tb = operr.get_w_traceback(space)
+        return space.call_function(w_meth, w_exc, w_val, w_tb)
+    else:
+        return space.call_method(w_yf, "send", w_inputvalue_or_err)
+
 
 ### ____________________________________________________________ ###
 
