@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 import py
+import pytest
 from pypy.interpreter.argument import (Arguments, ArgErr, ArgErrUnknownKwds,
-        ArgErrMultipleValues, ArgErrMissing, ArgErrTooMany, ArgErrTooManyMethod)
+        ArgErrMultipleValues, ArgErrMissing, ArgErrTooMany, ArgErrTooManyMethod,
+        ArgErrPosonlyAsKwds)
 from pypy.interpreter.signature import Signature
 from pypy.interpreter.error import OperationError
 
@@ -47,6 +49,18 @@ class TestSignature(object):
         assert sig.find_argname("d") == -1
         assert sig.find_argname("kwonly") == 3
 
+    def test_posonly(self):
+        sig = Signature(["a", "b", "c"], posonlyargnames=["x", "y", "z"])
+        # posonly come first
+        assert sig.find_argname("x") == 0
+        assert sig.find_argname("y") == 1
+        assert sig.find_argname("z") == 2
+        assert sig.find_argname("a") == 3
+        assert sig.find_argname("b") == 4
+        assert sig.find_argname("c") == 5
+        assert sig.find_argname("d") == -1
+
+
 class dummy_wrapped_dict(dict):
     def __nonzero__(self):
         raise NotImplementedError
@@ -138,6 +152,7 @@ class DummySpace(object):
         class Type:
             def getname(self, space):
                 return type(obj).__name__
+            name = type(obj).__name__
         return Type()
 
 
@@ -332,15 +347,17 @@ class TestArgumentsNormal(object):
 
     def test_duplicate_kwds(self):
         space = DummySpace()
-        excinfo = py.test.raises(OperationError, Arguments, space, [], ["a"],
-                                 [1], w_starstararg={"a": 2})
+        with pytest.raises(OperationError) as excinfo:
+            Arguments(space, [], ["a"], [1], w_starstararg={"a": 2}, fnname_parens="foo()")
         assert excinfo.value.w_type is TypeError
+        assert excinfo.value.get_w_value(space) == "foo() got multiple values for keyword argument 'a'"
 
     def test_starstararg_wrong_type(self):
         space = DummySpace()
-        excinfo = py.test.raises(OperationError, Arguments, space, [], ["a"],
-                                 [1], w_starstararg="hello")
+        with pytest.raises(OperationError) as excinfo:
+            Arguments(space, [], ["a"], [1], w_starstararg="hello", fnname_parens="bar()")
         assert excinfo.value.w_type is TypeError
+        assert excinfo.value.get_w_value(space) == "bar() argument after ** must be a mapping, not str"
 
     def test_unwrap_error(self):
         space = DummySpace()
@@ -353,12 +370,12 @@ class TestArgumentsNormal(object):
             return bytes(w, 'utf-8')
         space.utf8_w = utf8_w
         space.text_w = utf8_w
-        excinfo = py.test.raises(OperationError, Arguments, space, [],
-                                 ["a"], [1], w_starstararg={None: 1})
+        with py.test.raises(OperationError) as excinfo:
+            Arguments(space, [], ["a"], [1], w_starstararg={None: 1}, fnname_parens="f1()")
         assert excinfo.value.w_type is TypeError
         assert excinfo.value._w_value is None
-        excinfo = py.test.raises(OperationError, Arguments, space, [],
-                                 ["a"], [1], w_starstararg={valuedummy: 1})
+        with py.test.raises(OperationError) as excinfo:
+            Arguments(space, [], ["a"], [1], w_starstararg={valuedummy: 1}, fnname_parens="f2()")
         assert excinfo.value.w_type is ValueError
         assert excinfo.value._w_value is None
 
@@ -435,9 +452,9 @@ class TestArgumentsNormal(object):
             raise FakeArgErr()
         args._match_signature = _match_signature
 
-
-        excinfo = py.test.raises(OperationError, args.parse_obj, "obj", "foo",
-                       Signature(["a", "b"], None, None))
+        with pytest.raises(OperationError) as excinfo:
+            args.parse_obj("obj", "foo",
+                           Signature(["a", "b"], None, None))
         assert excinfo.value.w_type is TypeError
         assert excinfo.value.get_w_value(space) == "foo() msg"
 
@@ -491,9 +508,9 @@ class TestArgumentsNormal(object):
         args._match_signature = _match_signature
 
 
-        excinfo = py.test.raises(OperationError, args.parse_into_scope,
-                                 "obj", [None, None], "foo",
-                                 Signature(["a", "b"], None, None))
+        with pytest.raises(OperationError) as excinfo:
+            args.parse_into_scope("obj", [None, None], "foo",
+                                  Signature(["a", "b"], None, None))
         assert excinfo.value.w_type is TypeError
         assert excinfo.value.get_w_value(space) == "foo() msg"
 
@@ -568,10 +585,31 @@ class TestArgumentsNormal(object):
             l = [None, None, None]
             args._match_signature(None, l, Signature(["a", "b"], None, "**"))
             assert l == [1, 2, {'c': 3}]
-        excinfo = py.test.raises(OperationError, Arguments, space, [], ["a"],
-                                 [1], w_starstararg=kwargs(["a"], [2]))
+        with pytest.raises(OperationError) as excinfo:
+            Arguments(space, [], ["a"],
+                      [1], w_starstararg=kwargs(["a"], [2]))
         assert excinfo.value.w_type is TypeError
+        assert excinfo.value.get_w_value(space) == "got multiple values for keyword argument 'a'"
 
+        with pytest.raises(OperationError) as excinfo:
+            Arguments(space, [], ["a"],
+                      [1], w_starstararg=kwargs(["a"], [2]), fnname_parens="foo()")
+        assert excinfo.value.w_type is TypeError
+        assert excinfo.value.get_w_value(space) == "foo() got multiple values for keyword argument 'a'"
+
+    def test_posonly(self):
+        space = DummySpace()
+        sig = Signature(["a", "b", "c"], posonlyargnames=["x", "y", "z"])
+
+        args = Arguments(space, [1, 2, 3, 4, 5, 6])
+        l = [None] * 6
+        args._match_signature(None, l, sig)
+        assert l == [1, 2, 3, 4, 5, 6]
+
+        args = Arguments(space, [1, 2, 3, 4, 5], ["c"], [6])
+        l = [None] * 6
+        args._match_signature(None, l, sig)
+        assert l == [1, 2, 3, 4, 5, 6]
 
 
 class TestErrorHandling(object):
@@ -671,20 +709,14 @@ class TestErrorHandling(object):
 
     def test_bad_type_for_star(self):
         space = self.space
-        try:
-            Arguments(space, [], w_stararg=space.wrap(42))
-        except OperationError as e:
-            msg = space.text_w(space.str(e.get_w_value(space)))
-            assert msg == "argument after * must be an iterable, not int"
-        else:
-            assert 0, "did not raise"
-        try:
-            Arguments(space, [], w_starstararg=space.wrap(42))
-        except OperationError as e:
-            msg = space.text_w(space.str(e.get_w_value(space)))
-            assert msg == "argument after ** must be a mapping, not int"
-        else:
-            assert 0, "did not raise"
+        with pytest.raises(OperationError) as excinfo:
+            Arguments(space, [], w_stararg=space.wrap(42), fnname_parens="f1()")
+        msg = space.text_w(excinfo.value.get_w_value(space))
+        assert msg == "f1() argument after * must be an iterable, not int"
+        with pytest.raises(OperationError) as excinfo:
+            Arguments(space, [], w_starstararg=space.wrap(42), fnname_parens="f2()")
+        msg = space.text_w(excinfo.value.get_w_value(space))
+        assert msg == "f2() argument after ** must be a mapping, not int"
 
     def test_dont_count_default_arguments(self):
         space = self.space
@@ -771,6 +803,17 @@ class TestErrorHandling(object):
         s = err.getmsg()
         assert s == "got multiple values for argument 'bla'"
 
+    def test_posonly_error(self):
+        space = DummySpace()
+        sig = Signature([], posonlyargnames=["x", "y", "z"])
+
+        with pytest.raises(ArgErrPosonlyAsKwds) as info:
+            args = Arguments(space, [1, 2, 3, 4, 5], ["x"], [6])
+            l = [None] * 6
+            args._match_signature(None, l, sig)
+        assert info.value.getmsg() == "got an unexpected keyword argument 'x'"
+
+
 class AppTestArgument:
     def test_error_message(self):
         exc = raises(TypeError, (lambda a, b=2: 0), b=3)
@@ -800,7 +843,7 @@ class AppTestArgument:
         exc = raises(TypeError, '(lambda *, kw: 0)(1, kw=3)')
         assert str(exc.value) == "<lambda>() takes 0 positional arguments but 1 positional argument (and 1 keyword-only argument) were given"
 
-    @py.test.mark.skipif("config.option.runappdirect")
+    @pytest.mark.pypy_only
     def test_error_message_method(self):
         class A(object):
             def f0():
@@ -817,14 +860,14 @@ class AppTestArgument:
         # does not contain the warning about missing self
         assert exc.value.args[0] == "f0() takes 0 positional arguments but 1 was given"
 
-    @py.test.mark.skipif("config.option.runappdirect")
     def test_error_message_module_function(self):
         import operator # use countOf because it's defined at applevel
         exc = raises(TypeError, lambda : operator.countOf(1, 2, 3))
-        # does not contain the warning about missing self
-        assert exc.value.args[0] == "countOf() takes 2 positional arguments but 3 were given"
+        # does not contain the warning
+        # 'Did you forget 'self' in the function definition?'
+        assert 'self' not in str(exc.value)
 
-    @py.test.mark.skipif("config.option.runappdirect")
+    @pytest.mark.pypy_only
     def test_error_message_bound_method(self):
         class A(object):
             def f0():
@@ -889,7 +932,7 @@ class AppTestArgument:
             pass
         e = raises(TypeError, "f(*42)")
         assert str(e.value).endswith(
-            "argument after * must be an iterable, not int")
+            "f() argument after * must be an iterable, not int")
         e = raises(TypeError, "f(*X())")
         assert str(e.value) == "myerror"
 
@@ -904,8 +947,10 @@ class AppTestArgument:
         def f(x, y):
             pass
         e = raises(TypeError, "f(y=2, **{3: 5}, x=6)")
-        assert "keywords must be strings" in str(e.value)
+        assert "f() keywords must be strings" in str(e.value)
         e = raises(TypeError, "f(y=2, **{'x': 5}, x=6)")
+        # CPython figures out the name here, by peeking around in the stack in
+        # BUILD_MAP_UNPACK_WITH_CALL. we don't, too messy
         assert "got multiple values for keyword argument 'x'" in str(e.value)
 
     def test_dict_subclass_with_weird_getitem(self):
@@ -923,3 +968,18 @@ class AppTestArgument:
             def test(**kwargs):
                 return kwargs
             assert test(**q) == {"foo": "bar"}
+
+    def test_issue2996_1(self): """
+        class Class:
+            def method(*args, a_parameter=None, **kwargs):
+                pass
+        Class().method(**{'a_parameter': 4})
+        """
+
+    def test_issue2996_2(self): """
+        class Foo:
+            def methhh(*args, offset=42):
+                return args, offset
+        foo = Foo()
+        assert foo.methhh(**{}) == ((foo,), 42)
+        """
