@@ -343,7 +343,6 @@ class TestPythonAstCompiler:
             assert ex.match(self.space, self.space.w_SyntaxError)
 
     def test_globals_warnings(self):
-        # also tests some other constructions that give a warning
         space = self.space
         w_mod = space.appexec((), '():\n import warnings\n return warnings\n') #sys.getmodule('warnings')
         w_filterwarnings = space.getattr(w_mod, space.wrap('filterwarnings'))
@@ -365,18 +364,6 @@ def wrong3():
     print x
     x = 2
     global x
-''', '''
-def wrong_listcomp():
-    return [(yield 42) for i in j]
-''', '''
-def wrong_gencomp():
-    return ((yield 42) for i in j)
-''', '''
-def wrong_dictcomp():
-    return {(yield 42):2 for i in j}
-''', '''
-def wrong_setcomp():
-    return {(yield 42) for i in j}
 '''):
 
             space.call_args(w_filterwarnings, filter_arg)
@@ -706,31 +693,26 @@ def test():
 
     def test_dont_inherit_flag(self):
         # this test checks that compile() don't inherit the __future__ flags
-        # of the hosting code. However, in Python3 we don't have any
-        # meaningful __future__ flag to check that (they are all enabled). The
-        # only candidate could be barry_as_FLUFL, but it's not implemented yet
-        # (and not sure it'll ever be)
-        py.test.skip("we cannot actually check the result of this test (see comment)")
+        # of the hosting code.
         space = self.space
         s1 = str(py.code.Source("""
-            from __future__ import division
-            exec(compile('x = 1/2', '?', 'exec', 0, 1))
+            from __future__ import barry_as_FLUFL
+            # not a syntax error inside the exec!
+            exec(compile('x = 1 != 2', '?', 'exec', 0, 1))
         """))
         w_result = space.appexec([space.wrap(s1)], """(s1):
             ns = {}
             exec(s1, ns)
             return ns['x']
         """)
-        assert space.float_w(w_result) == 0
+        assert space.is_true(w_result)
 
     def test_dont_inherit_across_import(self):
-        # see the comment for test_dont_inherit_flag
-        py.test.skip("we cannot actually check the result of this test (see comment)")
         from rpython.tool.udir import udir
-        udir.join('test_dont_inherit_across_import.py').write('x = 1/2\n')
+        udir.join('test_dont_inherit_across_import.py').write('x = 1 != 2\n')
         space = self.space
         s1 = str(py.code.Source("""
-            from __future__ import division
+            from __future__ import barry_as_FLUFL
             from test_dont_inherit_across_import import x
         """))
         w_result = space.appexec([space.wrap(str(udir)), space.wrap(s1)],
@@ -738,13 +720,14 @@ def test():
             import sys
             copy = sys.path[:]
             sys.path.insert(0, udir)
+            ns = {}
             try:
-                exec s1
+                exec(s1, ns)
             finally:
                 sys.path[:] = copy
-            return x
+            return ns['x']
         """)
-        assert space.float_w(w_result) == 0
+        assert space.is_true(w_result)
 
     def test_filename_in_syntaxerror(self):
         e = py.test.raises(OperationError, self.compiler.compile, """if 1:
@@ -824,6 +807,13 @@ with somtehing as stuff:
         sig = cpython_code_signature(co)
         assert sig == Signature(['a', 'b'], None, 'kwargs', ['m', 'n'])
 
+        # a variant with varargname, which was buggy before issue2996
+        snippet = 'def f(*args, offset=42): pass'
+        containing_co = self.compiler.compile(snippet, '<string>', 'single', 0)
+        co = find_func(containing_co)
+        sig = cpython_code_signature(co)
+        assert sig == Signature([], 'args', None, ['offset'])
+
 
 class AppTestCompiler(object):
 
@@ -910,9 +900,11 @@ class AppTestCompiler(object):
         code = "from __future__ import barry_as_FLUFL; 2 {0} 3"
         compile(code.format('<>'), '<BDFL test>', 'exec',
                 __future__.CO_FUTURE_BARRY_AS_BDFL)
-        raises(SyntaxError, compile, code.format('!='),
+        with raises(SyntaxError) as excinfo:
+            compile(code.format('!='),
                '<FLUFL test>', 'exec',
                __future__.CO_FUTURE_BARRY_AS_BDFL)
+        assert excinfo.value.msg == "with Barry as BDFL, use '<>' instead of '!='"
 
     def test_guido_as_bdfl(self):
         # from test_flufl.py :-)
@@ -1015,6 +1007,27 @@ class AppTestCompiler(object):
         assert excinfo.value.lineno == 2
         assert excinfo.value.offset == 14
 
+    def test_zeros_not_mixed_in_lambdas(self):
+        import math
+        code = compile("x = lambda: -0.0; y = lambda: 0.0", "<test>", "exec")
+        consts = code.co_consts
+        x, _, y, z = consts
+        assert isinstance(x, type(code)) and isinstance(y, type(code))
+        assert x is not y
+        assert x != y
+
+    @py.test.mark.skipif('config.option.runappdirect')
+    def test_dont_share_lambdas(self):
+        # the two lambdas's codes aren't shared (CPython does that but it's
+        # completely pointless: it only applies to identical lambdas that are
+        # defined on the same line)
+        code = compile("x = lambda: 0; y = lambda: 0", "<test>", "exec")
+        consts = code.co_consts
+        x, _, y, z = consts
+        assert isinstance(x, type(code)) and isinstance(y, type(code))
+        assert x is not y
+        assert x == y
+
     def test_dict_and_set_literal_order(self):
         x = 1
         l1 = list({1:'a', 3:'b', 2:'c', 4:'d'})
@@ -1513,3 +1526,9 @@ class AppTestExceptions:
         assert eval(code) == u'# -*- coding: ascii -*-\n\xa4'
 
 
+
+    def test_asterror_has_line_without_file(self):
+        code = u"print(1)\na/2 = 5\n"
+        with raises(SyntaxError) as excinfo:
+            compile(code, 'not a file!', 'exec')
+        assert excinfo.value.text == "a/2 = 5\n"
