@@ -105,6 +105,17 @@ def strategy(space, w_obj):
         raise oefmt(space.w_TypeError, "expecting dict or list or set object")
     return space.newtext(name)
 
+def get_console_cp(space):
+    """get_console_cp()
+
+    Return the console and console output code page (windows only)
+    """
+    from rpython.rlib import rwin32    # Windows only
+    return space.newtuple([
+        space.newtext('cp%d' % rwin32.GetConsoleCP()),
+        space.newtext('cp%d' % rwin32.GetConsoleOutputCP()),
+        ])
+
 @unwrap_spec(sizehint=int)
 def resizelist_hint(space, w_list, sizehint):
     """ Reallocate the underlying storage of the argument list to sizehint """
@@ -211,3 +222,29 @@ def pyos_inputhook(space):
         return      # cpyext not imported yet, ignore
     from pypy.module.cpyext.api import invoke_pyos_inputhook
     invoke_pyos_inputhook(space)
+
+def utf8content(space, w_u):
+    """ Given a unicode string u, return it's internal byte representation.
+    Useful for debugging only. """
+    from pypy.objspace.std.unicodeobject import W_UnicodeObject
+    if type(w_u) is not W_UnicodeObject:
+        raise oefmt(space.w_TypeError, "expected unicode string, got %T", w_u)
+    return space.newbytes(w_u._utf8)
+
+def set_exc_info(space, w_type, w_value, w_traceback=None):
+    ec = space.getexecutioncontext()
+    ec.set_sys_exc_info3(w_type, w_value, w_traceback)
+
+def get_contextvar_context(space):
+    ec = space.getexecutioncontext()
+    context = ec.contextvar_context
+    if context:
+        return context
+    else:
+        return space.w_None
+
+def set_contextvar_context(space, w_obj):
+    ec = space.getexecutioncontext()
+    ec.contextvar_context = w_obj
+    return space.w_None
+
