@@ -1,3 +1,4 @@
+# encoding: utf-8
 
 class AppTestMagic:
     spaceconfig = dict(usemodules=['__pypy__'])
@@ -92,3 +93,21 @@ def f():
         terr = TypeError("hello world")
         set_exc_info(TypeError, terr, tb)
         assert sys.exc_info()[2] is tb
+
+    def test_utf8_content(self):
+        from __pypy__ import utf8content
+        assert utf8content(u"a") == b"a"
+        assert utf8content(u"\xe4") == b'\xc3\xa4'
+
+    def test_get_set_contextvar_context(self):
+        from __pypy__ import get_contextvar_context, set_contextvar_context
+        context = get_contextvar_context()
+        try:
+            set_contextvar_context(1)
+            assert get_contextvar_context() == 1
+            set_contextvar_context(5)
+            assert get_contextvar_context() == 5
+
+        finally:
+            set_contextvar_context(context)
+
