@@ -9,7 +9,7 @@ from pypy.interpreter.typedef import (
 from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from pypy.interpreter.buffer import SimpleView
 
-from rpython.rlib.buffer import ByteBuffer, SubBuffer
+from rpython.rlib.buffer import ByteBuffer, RawByteBuffer, SubBuffer
 from rpython.rlib.rstring import StringBuilder
 from rpython.rlib.rarithmetic import r_longlong, intmask
 from rpython.rlib import rposix
@@ -85,7 +85,7 @@ Returns None if the underlying raw stream was open in non-blocking
 mode and no data is available at the moment."""
         self._unsupportedoperation(space, "read")
 
-    def read1_w(self, space, w_size):
+    def read1_w(self, space, w_size=None):
         """Read and return up to n bytes, with at most one read() call
 to the underlying raw stream. A short result does not imply
 that EOF is imminent.
@@ -131,7 +131,7 @@ state."""
                         "%s() returned too much data: "
                         "%d bytes requested, %d returned",
                         methodname, length, len(data))
-        rwbuffer.setslice(0, data)
+        self.output_slice(space, rwbuffer, 0, data)
         return space.newint(len(data))
 
 W_BufferedIOBase.typedef = TypeDef(
@@ -197,7 +197,14 @@ class BufferedMixin:
             raise oefmt(space.w_ValueError,
                         "buffer size must be strictly positive")
 
-        self.buffer = ByteBuffer(self.buffer_size)
+        if space.config.translation.split_gc_address_space:
+            # When using split GC address space, it is not possible to get the
+            # raw address of a GC buffer. Therefore we use a buffer backed by
+            # raw memory.
+            self.buffer = RawByteBuffer(self.buffer_size)
+        else:
+            # TODO: test whether using the raw buffer is faster
+            self.buffer = ByteBuffer(self.buffer_size)
 
         self.lock = TryLock(space)
 
@@ -319,7 +326,6 @@ class BufferedMixin:
         with self.lock:
             if self.writable:
                 self._writer_flush_unlocked(space)
-                self._writer_reset_buf()
 
             if whence == 1:
                 pos -= self._raw_offset()
@@ -371,6 +377,7 @@ class BufferedMixin:
 
     def _writer_flush_unlocked(self, space):
         if self.write_end == -1 or self.write_pos == self.write_end:
+            self._writer_reset_buf()
             return
         # First, rewind
         rewind = self._raw_offset() + (self.pos - self.write_pos)
@@ -435,6 +442,7 @@ class BufferedMixin:
     @unwrap_spec(w_size = WrappedDefault(None))
     def truncate_w(self, space, w_size):
         self._check_init(space)
+        self._check_closed(space, "truncate of closed file")
         with self.lock:
             if self.writable:
                 self._flush_and_rewind_unlocked(space)
@@ -492,11 +500,11 @@ class BufferedMixin:
             return space.newbytes(data)
 
     @unwrap_spec(size=int)
-    def read1_w(self, space, size):
+    def read1_w(self, space, size=-1):
         self._check_closed(space, "read of closed file")
 
         if size < 0:
-            raise oefmt(space.w_ValueError, "read length must be positive")
+            size = self.buffer_size
         if size == 0:
             return space.newbytes("")
 
@@ -609,7 +617,7 @@ class BufferedMixin:
         remaining = n
         written = 0
         if current_size:
-            result_buffer.setslice(
+            self.output_slice(space, result_buffer,
                 written, self.buffer[self.pos:self.pos + current_size])
             remaining -= current_size
             written += current_size
@@ -654,7 +662,7 @@ class BufferedMixin:
             if remaining > 0:
                 if size > remaining:
                     size = remaining
-                result_buffer.setslice(
+                self.output_slice(space, result_buffer,
                     written, self.buffer[self.pos:self.pos + size])
                 self.pos += size
                 written += size
@@ -713,7 +721,10 @@ class BufferedMixin:
 
             while True:
                 self._reader_reset_buf()
-                have = self._fill_buffer(space)
+                try:
+                    have = self._fill_buffer(space)
+                except BlockingIOError:
+                    have = 0
                 if have == 0:
                     break
                 if limit >= 0 and have > limit:
@@ -721,8 +732,7 @@ class BufferedMixin:
                 pos = 0
                 found = False
                 while pos < have:
-                    # 'buffer.data[]' instead of 'buffer[]' because RPython...
-                    c = self.buffer.data[pos]
+                    c = self.buffer.getitem(pos)
                     pos += 1
                     if c == '\n':
                         self.pos = pos
@@ -749,11 +759,12 @@ class BufferedMixin:
             self.read_end = self.pos
 
     def write_w(self, space, w_data):
-        self._check_closed(space, "write to closed file")
+        self._check_init(space)
         data = space.charbuf_w(w_data)
         size = len(data)
 
         with self.lock:
+            self._check_closed(space, "write to closed file")
             if (not (self.readable and self.read_end != -1) and
                 not (self.writable and self.write_end != -1)):
                 self.pos = 0
@@ -780,8 +791,7 @@ class BufferedMixin:
                     self._reader_reset_buf()
                 # Make some place by shifting the buffer
                 for i in range(self.write_pos, self.write_end):
-                    # XXX: messing with buffer internals
-                    self.buffer.data[i - self.write_pos] = self.buffer.data[i]
+                    self.buffer.setitem(i - self.write_pos, self.buffer.getitem(i))
                 self.write_end -= self.write_pos
                 self.raw_pos -= self.write_pos
                 newpos = self.pos - self.write_pos
@@ -884,12 +894,14 @@ class BufferedReaderMixin(BufferedMixin):
         with self.lock:
             have = self._readahead()
             if have >= length:
-                rwbuffer.setslice(0, self.buffer[self.pos:self.pos + length])
+                self.output_slice(space, rwbuffer,
+                    0, self.buffer[self.pos:self.pos + length])
                 self.pos += length
                 return space.newint(length)
             written = 0
             if have > 0:
-                rwbuffer.setslice(0, self.buffer[self.pos:self.read_end])
+                self.output_slice(space, rwbuffer,
+                    0, self.buffer[self.pos:self.read_end])
                 written = have
 
             while written < length:
@@ -917,7 +929,8 @@ class BufferedReaderMixin(BufferedMixin):
                         break
                     endpos = min(have, length - written)
                     assert endpos >= 0
-                    rwbuffer.setslice(written, self.buffer[0:endpos])
+                    self.output_slice(space, rwbuffer,
+                        written, self.buffer[0:endpos])
                     written += endpos
                     self.pos = endpos
                 if read_once:
