@@ -5,7 +5,7 @@ from rpython.rlib.objectmodel import specialize
 from rpython.rlib.rarithmetic import intmask
 from rpython.rlib.rsocket import (
     RSocket, AF_INET, SOCK_STREAM, SocketError, SocketErrorWithErrno,
-    RSocketError, SOMAXCONN
+    RSocketError, SOMAXCONN, HAS_SO_PROTOCOL,
 )
 from rpython.rtyper.lltypesystem import lltype, rffi
 
@@ -87,6 +87,16 @@ def fill_from_object(addr, space, w_address):
     else:
         raise NotImplementedError
 
+
+if HAS_SO_PROTOCOL:
+    def get_so_protocol(fd):
+        from rpython.rlib import _rsocket_rffi as _c
+        return rsocket.getsockopt_int(fd, _c.SOL_SOCKET, _c.SO_PROTOCOL)
+else:
+    def get_so_protocol(fd):
+        return -1
+
+
 def idna_converter(space, w_host):
     # Converts w_host to a byte string.  Similar to encode_idna()
     # but accepts more types and refuses NULL bytes.
@@ -200,12 +210,19 @@ class W_Socket(W_Root):
 
     @unwrap_spec(family=int, type=int, proto=int,
                  w_fileno=WrappedDefault(None))
-    def descr_init(self, space, family=AF_INET, type=SOCK_STREAM, proto=0,
+    def descr_init(self, space, family=-1, type=-1, proto=-1,
                    w_fileno=None):
+        from rpython.rlib.rsocket import _c
+        if space.is_w(w_fileno, space.w_None):
+            if family == -1:
+                family = AF_INET
+            if type == -1:
+                type = SOCK_STREAM
+            if proto == -1:
+                proto = 0
         try:
             if not space.is_w(w_fileno, space.w_None):
                 if _WIN32 and space.isinstance_w(w_fileno, space.w_bytes):
-                    from rpython.rlib.rsocket import _c
                     # it is possible to pass some bytes representing a socket
                     # in the file descriptor object on winodws
                     fdobj = space.bytes_w(w_fileno)
@@ -225,12 +242,16 @@ class W_Socket(W_Root):
                     finally:
                         lltype.free(info_charptr, flavor='raw')
                 else:
-                    sock = RSocket(family, type, proto,
-                                   fd=space.c_filedescriptor_w(w_fileno))
+                    fd = space.c_filedescriptor_w(w_fileno)
+                    if family == -1:
+                        family = rsocket.get_socket_family(fd)
+                    if type == -1:
+                        type = rsocket.getsockopt_int(fd, _c.SOL_SOCKET, _c.SO_TYPE)
+                    if proto == -1:
+                        proto = get_so_protocol(fd)
+                    sock = RSocket(family, type, proto, fd=fd)
             else:
                 sock = RSocket(family, type, proto, inheritable=False)
-            # On Python3.6+, the SOCK_CLOEXEC flag (if preset) is added
-            sock.type = type
             W_Socket.__init__(self, space, sock)
         except SocketError as e:
             raise converted_error(space, e)
@@ -416,12 +437,21 @@ class W_Socket(W_Root):
 
         Returns the timeout in floating seconds associated with socket
         operations. A timeout of None indicates that timeouts on socket
+        operations are disabled.
         """
         timeout = self.sock.gettimeout()
         if timeout < 0.0:
             return space.w_None
         return space.newfloat(timeout)
 
+    def getblocking_w(self, space):
+        """getblocking()
+
+        Returns True if socket is in blocking mode, or False if it
+        is in non-blocking mode.
+        """
+        return space.newbool(self.sock.gettimeout() != 0.0)
+
     @unwrap_spec(backlog="c_int")
     def listen_w(self, space, backlog=min(SOMAXCONN, 128)):
         """listen(backlog)
@@ -1036,6 +1066,7 @@ getpeername getsockname getsockopt gettimeout listen
 recv recvfrom send sendall sendto setblocking
 setsockopt settimeout shutdown _reuse _drop
 recv_into recvfrom_into
+getblocking
 """.split()
 if hasattr(rsocket._c, 'WSAIoctl'):
     socketmethodnames.append('ioctl')
@@ -1102,3 +1133,11 @@ if_indextoname(index) -- return the corresponding interface name
     family = GetSetProperty(W_Socket.get_family_w),
     **socketmethods
     )
+
+@unwrap_spec(fd=int)
+def close(space, fd):
+    from rpython.rlib import _rsocket_rffi as _c
+    res = _c.socketclose(fd)
+    if res:
+        converted_error(space, rsocket.last_error())
+
