@@ -114,6 +114,18 @@ class AppTestArray(object):
             assert a[2] == 2.5
             assert len(a) == len(values)
 
+    def test_nan(self):
+        for tc in 'fd':
+            a = self.array(tc, [float('nan')])
+            b = self.array(tc, [float('nan')])
+            assert not a == b
+            assert a != b
+            assert not a > b
+            assert not a >= b
+            assert not a < b
+            assert not a <= b
+            assert a.count(float('nan')) == 0
+
     def test_itemsize(self):
         for t in 'bB':
             assert(self.array(t).itemsize >= 1)
@@ -864,25 +876,25 @@ class AppTestArray(object):
             def extend(self, lst):
                 self.append(10)
 
-        assert repr(mya('u', 'hi')) == "array('u', 'hi')"
-        assert repr(mya('i', [1, 2, 3])) == "array('i', [1, 2, 3])"
-        assert repr(mya('i', (1, 2, 3))) == "array('i', [1, 2, 3])"
+        assert repr(mya('u', 'hi')) == "mya('u', 'hi')"
+        assert repr(mya('i', [1, 2, 3])) == "mya('i', [1, 2, 3])"
+        assert repr(mya('i', (1, 2, 3))) == "mya('i', [1, 2, 3])"
 
         a = mya('i')
         a.fromlist([1, 2, 3])
-        assert repr(a) == "array('i', [7])"
+        assert repr(a) == "mya('i', [7])"
 
         a = mya('b')
         a.fromstring(b'hi')
-        assert repr(a) == "array('b', [8])"
+        assert repr(a) == "mya('b', [8])"
 
         a = mya('u')
         a.fromunicode('hi')
-        assert repr(a) == "array('u', '9')"
+        assert repr(a) == "mya('u', '9')"
 
         a = mya('i')
         a.extend([1, 2, 3])
-        assert repr(a) == "array('i', [10])"
+        assert repr(a) == "mya('i', [10])"
 
     def test_override_to(self):
         class mya(self.array):
@@ -899,9 +911,9 @@ class AppTestArray(object):
         assert mya('u', 'hi').tobytes() == 'str'
         assert mya('u', 'hi').tounicode() == 'unicode'
 
-        assert repr(mya('u', 'hi')) == "array('u', 'hi')"
-        assert repr(mya('i', [1, 2, 3])) == "array('i', [1, 2, 3])"
-        assert repr(mya('i', (1, 2, 3))) == "array('i', [1, 2, 3])"
+        assert repr(mya('u', 'hi')) == "mya('u', 'hi')"
+        assert repr(mya('i', [1, 2, 3])) == "mya('i', [1, 2, 3])"
+        assert repr(mya('i', (1, 2, 3))) == "mya('i', [1, 2, 3])"
 
     def test_unicode_outofrange(self):
         input_unicode = u'\x01\u263a\x00\ufeff'
@@ -1111,6 +1123,12 @@ class AppTestArray(object):
         struct.pack_into('>H', view, 1, 0x1234)
         assert a.tobytes() == b'ab\x12\x34ef'
 
+    def test_subclass_repr(self):
+        import array
+        class subclass(self.array):
+            pass
+        assert repr(subclass('i')) == "subclass('i')"
+
 
 class AppTestArrayReconstructor:
     spaceconfig = dict(usemodules=('array', 'struct'))
