@@ -12,7 +12,7 @@ from rpython.rtyper.annlowlevel import llhelper
 from rpython.rlib.objectmodel import we_are_translated, keepalive_until_here
 from rpython.rlib.objectmodel import dont_inline
 from rpython.rlib.rfile import (FILEP, c_fread, c_fclose, c_fwrite,
-        c_fdopen, c_fileno,
+        c_fdopen, c_fileno, c_ferror,
         c_fopen)# for tests
 from rpython.rlib import jit, rutf8
 from rpython.translator import cdir
@@ -35,7 +35,6 @@ from rpython.rlib.entrypoint import entrypoint_lowlevel
 from rpython.rlib.rposix import FdValidator
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rlib.objectmodel import specialize
-from pypy.module import exceptions
 from pypy.module.exceptions import interp_exceptions
 from rpython.tool.sourcetools import func_with_new_name
 from rpython.rtyper.lltypesystem.lloperation import llop
@@ -120,6 +119,11 @@ def feof(fp):
     with FdValidator(c_fileno(fp)):
         return _feof(fp)
 
+_ferror = rffi.llexternal('ferror', [FILEP], rffi.INT)
+def ferror(fp):
+    with FdValidator(c_fileno(fp)):
+        return _ferror(fp)
+
 pypy_decl = 'pypy_decl.h'
 udir.join(pypy_decl).write("/* Will be filled later */\n")
 udir.join('pypy_structmember_decl.h').write("/* Will be filled later */\n")
@@ -132,7 +136,7 @@ METH_COEXIST METH_STATIC METH_CLASS Py_TPFLAGS_BASETYPE Py_MAX_FMT
 METH_NOARGS METH_VARARGS METH_KEYWORDS METH_O
 Py_TPFLAGS_HEAPTYPE
 Py_LT Py_LE Py_EQ Py_NE Py_GT Py_GE Py_MAX_NDIMS
-Py_CLEANUP_SUPPORTED
+Py_CLEANUP_SUPPORTED PyBUF_READ
 PyBUF_FORMAT PyBUF_ND PyBUF_STRIDES PyBUF_WRITABLE PyBUF_SIMPLE PyBUF_WRITE
 """.split()
 
@@ -211,19 +215,7 @@ CANNOT_FAIL = CannotFail()
 # Handling of the GIL
 # -------------------
 #
-# We add a global variable 'cpyext_glob_tid' that contains a thread
-# id.  Invariant: this variable always contain 0 when the PyPy GIL is
-# released.  It should also contain 0 when regular RPython code
-# executes.  In non-cpyext-related code, it will thus always be 0.
-# When cpyext-related C code runs, it contains the thread id (usually)
-# or the value -1 (only for state.C.PyXxx() functions which are short-
-# running and should not themselves release the GIL).
-#
-# **make_generic_cpy_call():** RPython to C, with the GIL held.  Before
-# the call, must assert that the global variable is 0 and set the
-# current thread identifier into the global variable.  After the call,
-# assert that the global variable still contains the current thread id,
-# and reset it to 0.
+# **make_generic_cpy_call():** RPython to C, with the GIL held.
 #
 # **make_wrapper():** C to RPython; by default assume that the GIL is
 # held, but accepts gil="acquire", "release", "around",
@@ -231,41 +223,30 @@ CANNOT_FAIL = CannotFail()
 #
 # When a wrapper() is called:
 #
-# * "acquire": assert that the GIL is not currently held, i.e. the
-#   global variable does not contain the current thread id (otherwise,
-#   deadlock!).  Acquire the PyPy GIL.  After we acquired it, assert
-#   that the global variable is 0 (it must be 0 according to the
-#   invariant that it was 0 immediately before we acquired the GIL,
-#   because the GIL was released at that point).
+# * "acquire": assert that the GIL is not currently held (otherwise,
+#   deadlock!).  Acquire the PyPy GIL.
 #
-# * gil=None: we hold the GIL already.  Assert that the current thread
-#   identifier is in the global variable, and replace it with 0.
+# * gil=None: we should hold the GIL already.  But check anyway, just
+#   in case.  Do the acquire/release if it was not acquired before
+#   (workaround "_auto" case).
 #
-# * "pygilstate_ensure": if the global variable contains the current
-#   thread id, replace it with 0 and set the extra arg to 0.  Otherwise,
+# * "pygilstate_ensure": if the GIL is already acquired,
+#   do nothing and set the extra arg to 0.  Otherwise,
 #   do the "acquire" and set the extra arg to 1.  Then we'll call
 #   pystate.py:PyGILState_Ensure() with this extra arg, which will do
 #   the rest of the logic.
 #
-# When a wrapper() returns, first assert that the global variable is
-# still 0, and then:
+# When a wrapper() returns:
 #
-# * "release": release the PyPy GIL.  The global variable was 0 up to
-#   and including at the point where we released the GIL, but afterwards
-#   it is possible that the GIL is acquired by a different thread very
-#   quickly.
+# * "release": release the PyPy GIL.
 #
-# * gil=None: we keep holding the GIL.  Set the current thread
-#   identifier into the global variable.
+# * gil=None: we keep holding the GIL in the normal case; we release it
+#   in the workaround "_auto" case.
 #
 # * "pygilstate_release": if the argument is PyGILState_UNLOCKED,
-#   release the PyPy GIL; otherwise, set the current thread identifier
-#   into the global variable.  The rest of the logic of
+#   release the PyPy GIL; otherwise, no-op.  The rest of the logic of
 #   PyGILState_Release() should be done before, in pystate.py.
 
-cpyext_glob_tid_ptr = lltype.malloc(rffi.CArray(lltype.Signed), 1,
-                                    flavor='raw', immortal=True, zero=True)
-
 
 cpyext_namespace = NameManager('cpyext_')
 
@@ -639,22 +620,24 @@ SYMBOLS_C = [
     'PyThread_ReInitTLS', 'PyThread_init_thread',
     'PyThread_start_new_thread',
 
-    'PyStructSequence_InitType', 'PyStructSequence_New',
-    'PyStructSequence_UnnamedField',
+    'PyStructSequence_InitType', 'PyStructSequence_InitType2',
+    'PyStructSequence_New', 'PyStructSequence_UnnamedField',
 
     'PyFunction_Type', 'PyMethod_Type', 'PyRange_Type', 'PyTraceBack_Type',
 
-    'Py_DebugFlag', 'Py_VerboseFlag', 'Py_InteractiveFlag', 'Py_InspectFlag',
+    'Py_DebugFlag', 'Py_VerboseFlag', 'Py_QuietFlag',
+    'Py_InteractiveFlag', 'Py_InspectFlag',
     'Py_OptimizeFlag', 'Py_NoSiteFlag', 'Py_BytesWarningFlag', 'Py_UseClassExceptionsFlag',
-    'Py_FrozenFlag', 'Py_TabcheckFlag', 'Py_UnicodeFlag', 'Py_IgnoreEnvironmentFlag',
-    'Py_DivisionWarningFlag', 'Py_DontWriteBytecodeFlag', 'Py_NoUserSiteDirectory',
-    '_Py_QnewFlag', 'Py_Py3kWarningFlag', 'Py_HashRandomizationFlag', '_Py_PackageContext',
+    'Py_FrozenFlag', 'Py_IgnoreEnvironmentFlag',
+    'Py_DontWriteBytecodeFlag', 'Py_NoUserSiteDirectory',
+    'Py_UnbufferedStdioFlag', 'Py_HashRandomizationFlag', 'Py_IsolatedFlag',
+    '_Py_PackageContext',
     'PyOS_InputHook',
 
     'PyMem_RawMalloc', 'PyMem_RawCalloc', 'PyMem_RawRealloc', 'PyMem_RawFree',
     'PyMem_Malloc', 'PyMem_Calloc', 'PyMem_Realloc', 'PyMem_Free',
     'PyObject_CallFinalizerFromDealloc',
-    '_PyTraceMalloc_Track', '_PyTraceMalloc_Untrack',
+    'PyTraceMalloc_Track', 'PyTraceMalloc_Untrack',
     'PyBytes_FromFormat', 'PyBytes_FromFormatV',
 
     'PyType_FromSpec',
@@ -664,6 +647,8 @@ SYMBOLS_C = [
     'PyObject_Init', 'PyObject_InitVar',
     'PyTuple_New', '_Py_Dealloc',
 ]
+if sys.platform == "win32":
+    SYMBOLS_C.append('Py_LegacyWindowsStdioFlag')
 TYPES = {}
 FORWARD_DECLS = []
 INIT_FUNCTIONS = []
@@ -689,8 +674,8 @@ def build_exported_objects():
     # PyExc_AttributeError, PyExc_OverflowError, PyExc_ImportError,
     # PyExc_NameError, PyExc_MemoryError, PyExc_RuntimeError,
     # PyExc_UnicodeEncodeError, PyExc_UnicodeDecodeError, ...
-    global all_exceptions
-    all_exceptions = list(exceptions.Module.interpleveldefs)
+    from pypy.module.exceptions.moduledef import Module as ExcModule
+    all_exceptions = list(ExcModule.interpleveldefs)
     for exc_name in all_exceptions:
         if exc_name in ('EnvironmentError', 'IOError', 'WindowsError'):
             # FIXME: aliases of OSError cause a clash of names via
@@ -956,10 +941,9 @@ def unexpected_exception(funcname, e, tb):
         pypy_debug_catch_fatal_exception()
         assert False
 
-def _restore_gil_state(pygilstate_release, gilstate, gil_release, _gil_auto, tid):
+def _restore_gil_state(pygilstate_release, gilstate, gil_release, _gil_auto):
     from rpython.rlib import rgil
     # see "Handling of the GIL" above
-    assert cpyext_glob_tid_ptr[0] == 0
     if pygilstate_release:
         from pypy.module.cpyext import pystate
         unlock = (gilstate == pystate.PyGILState_UNLOCKED)
@@ -967,8 +951,6 @@ def _restore_gil_state(pygilstate_release, gilstate, gil_release, _gil_auto, tid
         unlock = gil_release or _gil_auto
     if unlock:
         rgil.release()
-    else:
-        cpyext_glob_tid_ptr[0] = tid
 
 
 def make_wrapper_second_level(space, argtypesw, restype,
@@ -982,8 +964,9 @@ def make_wrapper_second_level(space, argtypesw, restype,
     gil_release = (gil == "release" or gil == "around")
     pygilstate_ensure = (gil == "pygilstate_ensure")
     pygilstate_release = (gil == "pygilstate_release")
+    pygilstate_check = (gil == "pygilstate_check")
     assert (gil is None or gil_acquire or gil_release
-            or pygilstate_ensure or pygilstate_release)
+            or pygilstate_ensure or pygilstate_release or pygilstate_check)
     expected_nb_args = len(argtypesw) + pygilstate_ensure
 
     if isinstance(restype, lltype.Ptr) and error_value == 0:
@@ -1003,37 +986,37 @@ def make_wrapper_second_level(space, argtypesw, restype,
         # inserted exactly here by the varargs specializer
 
         # see "Handling of the GIL" above (careful, we don't have the GIL here)
-        tid = rthread.get_or_make_ident()
         _gil_auto = False
-        if gil_auto_workaround and cpyext_glob_tid_ptr[0] != tid:
-            # replace '-1' with the real tid, now that we have the tid
-            if cpyext_glob_tid_ptr[0] == -1:
-                cpyext_glob_tid_ptr[0] = tid
-            else:
-                _gil_auto = True
+        if gil_auto_workaround and not rgil.am_I_holding_the_GIL():
+            _gil_auto = True
         if _gil_auto or gil_acquire:
-            if cpyext_glob_tid_ptr[0] == tid:
+            if gil_acquire and rgil.am_I_holding_the_GIL():
                 deadlock_error(pname)
             rgil.acquire()
-            assert cpyext_glob_tid_ptr[0] == 0
+            if gil_auto_workaround:
+                # while we're in workaround-land, detect when a regular PyXxx()
+                # function is invoked at .so load-time, e.g. by a C++ global
+                # variable with an initializer, and in this case make sure we
+                # initialize things.
+                space.fromcache(State).make_sure_cpyext_is_imported()
         elif pygilstate_ensure:
-            if cpyext_glob_tid_ptr[0] == tid:
-                cpyext_glob_tid_ptr[0] = 0
+            if rgil.am_I_holding_the_GIL():
                 args += (pystate.PyGILState_LOCKED,)
             else:
                 rgil.acquire()
                 args += (pystate.PyGILState_UNLOCKED,)
+        elif pygilstate_check:
+            result = rgil.am_I_holding_the_GIL()
+            return rffi.cast(restype, result)
         else:
-            if cpyext_glob_tid_ptr[0] != tid:
+            if not rgil.am_I_holding_the_GIL():
                 no_gil_error(pname)
-            cpyext_glob_tid_ptr[0] = 0
         if pygilstate_release:
             gilstate = rffi.cast(lltype.Signed, args[-1])
         else:
             gilstate = pystate.PyGILState_IGNORE
 
-        rffi.stackcounter.stacks_counter += 1
-        llop.gc_stack_bottom(lltype.Void)   # marker for trackgcroot.py
+        llop.gc_stack_bottom(lltype.Void)   # marker to enter RPython from C
         retval = fatal_value
         boxed_args = ()
         tb = None
@@ -1105,14 +1088,13 @@ def make_wrapper_second_level(space, argtypesw, restype,
 
         except Exception as e:
             unexpected_exception(pname, e, tb)
-            _restore_gil_state(pygilstate_release, gilstate, gil_release, _gil_auto, tid)
+            _restore_gil_state(pygilstate_release, gilstate, gil_release, _gil_auto)
             state.check_and_raise_exception(always=True)
             return fatal_value
 
         assert lltype.typeOf(retval) == restype
-        rffi.stackcounter.stacks_counter -= 1
 
-        _restore_gil_state(pygilstate_release, gilstate, gil_release, _gil_auto, tid)
+        _restore_gil_state(pygilstate_release, gilstate, gil_release, _gil_auto)
         return retval
 
     wrapper_second_level._dont_inline_ = True
@@ -1192,7 +1174,7 @@ def attach_c_functions(space, eci, prefix):
         '_PyPy_get_PyOS_InputHook', [], FUNCPTR,
         compilation_info=eci, _nowrapper=True)
     state.C.tuple_new = rffi.llexternal(
-        'tuple_new', [PyTypeObjectPtr, PyObject, PyObject], PyObject,
+        '_PyPy_tuple_new', [PyTypeObjectPtr, PyObject, PyObject], PyObject,
         compilation_info=eci, _nowrapper=True)
 
 def init_function(func):
@@ -1238,11 +1220,6 @@ def build_bridge(space):
     struct PyPyAPI* pypyAPI = &_pypyAPI;
     """ % dict(members=structmembers)
 
-    global_objects = []
-    for name in all_exceptions:
-        global_objects.append('PyTypeObject _PyExc_%s;' % name)
-    global_code = '\n'.join(global_objects)
-
     prologue = ("#include <Python.h>\n" +
                 "#include <structmember.h>\n" +
                 "#include <marshal.h>\n" +
@@ -1250,7 +1227,6 @@ def build_bridge(space):
                 "#include <src/thread.c>\n")
     code = (prologue +
             struct_declaration_code +
-            global_code +
             '\n' +
             '\n'.join(functions))
 
@@ -1506,6 +1482,7 @@ separate_module_files = [source_dir / "varargwrapper.c",
                          source_dir / "pythread.c",
                          source_dir / "missing.c",
                          source_dir / "pymem.c",
+                         source_dir / "pytime.c",
                          source_dir / "bytesobject.c",
                          source_dir / "import.c",
                          source_dir / "_warnings.c",
@@ -1513,7 +1490,13 @@ separate_module_files = [source_dir / "varargwrapper.c",
                          source_dir / "object.c",
                          source_dir / "typeobject.c",
                          source_dir / "tupleobject.c",
+                         source_dir / "sliceobject.c",
                          ]
+if WIN32:
+    separate_module_files.append(source_dir / "pythread_nt.c")
+else:
+    separate_module_files.append(source_dir / "pythread_posix.c")
+
 
 def build_eci(code, use_micronumpy=False, translating=False):
     "NOT_RPYTHON"
@@ -1589,7 +1572,6 @@ def setup_micronumpy(space):
         return use_micronumpy
     # import registers api functions by side-effect, we also need HEADER
     from pypy.module.cpyext.ndarrayobject import HEADER
-    global separate_module_files
     register_global("PyArray_Type",
         'PyTypeObject*',  "space.gettypeobject(W_NDimArray.typedef)",
         header=HEADER)
@@ -1741,8 +1723,8 @@ def create_cpyext_module(space, w_spec, name, path, dll, initptr):
     from rpython.rlib import rdynload
     from pypy.module.cpyext.pyobject import get_w_obj_and_decref
 
-    space.getbuiltinmodule("cpyext")    # mandatory to init cpyext
     state = space.fromcache(State)
+    state.make_sure_cpyext_is_imported()
     w_mod = state.find_extension(name, path)
     if w_mod is not None:
         rdynload.dlclose(dll)
@@ -1859,18 +1841,14 @@ def make_generic_cpy_call(FT, expect_null, convert_result):
             boxed_args += (arg,)
             to_decref += (_pyobj,)
 
-        # see "Handling of the GIL" above
-        tid = rthread.get_ident()
-        assert cpyext_glob_tid_ptr[0] == 0
-        cpyext_glob_tid_ptr[0] = tid
-
-        preexist_error = PyErr_Occurred(space)
+        if is_PyObject(RESULT_TYPE):
+            preexist_error = PyErr_Occurred(space)
+        else:
+            preexist_error = "this is not used"
         try:
             # Call the function
             result = call_external_function(func, *boxed_args)
         finally:
-            assert cpyext_glob_tid_ptr[0] == tid
-            cpyext_glob_tid_ptr[0] = 0
             for i, ARG in unrolling_arg_types:
                 # note that this loop is nicely unrolled statically by RPython
                 _pyobj = to_decref[i]
