@@ -243,14 +243,6 @@ def PyDescr_IsData(space, descr):
 def PyWrapper_New(space, w_d, w_self):
     raise NotImplementedError
 
-@cpython_api([PyObject, PyObject], PyObject)
-def PyDict_GetItemWithError(space, p, key):
-    """Variant of PyDict_GetItem() that does not suppress
-    exceptions. Return NULL with an exception set if an exception
-    occurred.  Return NULL without an exception set if the key
-    wasn't present."""
-    raise NotImplementedError
-
 @cpython_api([PyObject, PyObject, rffi.INT_real], rffi.INT_real, error=-1)
 def PyDict_MergeFromSeq2(space, a, seq2, override):
     """Update or merge into dictionary a, from the key-value pairs in seq2.
@@ -1197,16 +1189,6 @@ def PyModule_GetFilenameObject(space, module):
     __file__ attribute.  If this is not defined, or if it is not a
     unicode string, raise SystemError and return NULL; otherwise return
     a reference to a PyUnicodeObject.
-    """
-    raise NotImplementedError
-
-@cpython_api([PyObject, rffi.INT_real], PyObject)
-def PyNumber_ToBase(space, n, base):
-    """Returns the integer n converted to base base as a string.  The base
-    argument must be one of 2, 8, 10, or 16.  For base 2, 8, or 16, the
-    returned string is prefixed with a base marker of '0b', '0o', or
-    '0x', respectively.  If n is not a Python int, it is converted with
-    PyNumber_Index() first."""
     raise NotImplementedError
 
 @cpython_api([PyFrameObject], rffi.INT_real, error=-1)
@@ -1334,16 +1316,6 @@ def PyType_ClearCache(space, ):
     raise NotImplementedError
 
 
-@cpython_api([PyTypeObjectPtr], lltype.Signed, error=CANNOT_FAIL)
-def PyType_GetFlags(space, type):
-    """Return the tp_flags member of type. This function is primarily
-    meant for use with Py_LIMITED_API; the individual flag bits are
-    guaranteed to be stable across Python releases, but access to
-    tp_flags itself is not part of the limited API.
-    """
-    raise NotImplementedError
-
-
 @cpython_api([], rffi.INT_real, error=-1)
 def PyUnicode_ClearFreeList(space, ):
     """Clear the free list. Return the total number of freed items."""
