@@ -3,6 +3,7 @@ import sys
 import StringIO
 
 from pypy.module.cpyext.state import State
+from pypy.module.cpyext.pyobject import make_ref
 from pypy.module.cpyext.test.test_api import BaseApiTest
 from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
 from rpython.rtyper.lltypesystem import rffi
@@ -98,6 +99,30 @@ class TestExceptions(BaseApiTest):
         assert recieved == ['ok']
         assert api.PyOS_InterruptOccurred()
 
+    def test_restore_traceback(self, space, api):
+        string = rffi.str2charp("spam and eggs")
+        api.PyErr_SetString(space.w_ValueError, string)
+
+        state = space.fromcache(State)
+        operror = state.clear_exception()
+
+        # Fake a traceback.
+        operror.set_traceback(space.w_True) # this doesn't really need to be a real traceback for this test.
+
+        w_type = operror.w_type
+        w_value = operror.get_w_value(space)
+        w_tb = operror.get_w_traceback(space)
+
+        assert not space.eq_w(w_tb, space.w_None)
+
+        api.PyErr_Restore(make_ref(space, w_type), make_ref(space, w_value), make_ref(space, w_tb))
+
+        operror = state.clear_exception()
+        w_tb_restored = operror.get_w_traceback(space)
+
+        assert space.eq_w(w_tb_restored, w_tb)
+        rffi.free_charp(string)
+
 class AppTestFetch(AppTestCpythonExtensionBase):
     def setup_class(cls):
         from pypy.interpreter.test.test_fsencode import get_special_char
@@ -239,8 +264,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename == "/path/to/file"
-        assert exc_info.value.errno == errno.EBADF
-        assert exc_info.value.strerror == os.strerror(errno.EBADF)
+        if self.runappdirect:
+            # untranslated the errno can get reset by the calls to ll2ctypes
+            assert exc_info.value.errno == errno.EBADF
+            assert exc_info.value.strerror == os.strerror(errno.EBADF)
 
         exc_info = raises(OSError, module.set_from_errno_special)
         assert exc_info.value.filename == "/path/to/%s" % (char, )
@@ -261,8 +288,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename is None
-        assert exc_info.value.errno == errno.EBADF
-        assert exc_info.value.strerror == os.strerror(errno.EBADF)
+        if self.runappdirect:
+            # untranslated the errno can get reset by the calls to ll2ctypes
+            assert exc_info.value.errno == errno.EBADF
+            assert exc_info.value.strerror == os.strerror(errno.EBADF)
 
     def test_SetFromErrnoWithFilenameObject__PyUnicode(self):
         import errno, os
@@ -280,8 +309,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename == "/path/to/file"
-        assert exc_info.value.errno == errno.EBADF
-        assert exc_info.value.strerror == os.strerror(errno.EBADF)
+        if self.runappdirect:
+            # untranslated the errno can get reset by the calls to ll2ctypes
+            assert exc_info.value.errno == errno.EBADF
+            assert exc_info.value.strerror == os.strerror(errno.EBADF)
 
     def test_SetFromErrnoWithFilenameObject__PyLong(self):
         import errno, os
@@ -299,8 +330,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename == 3
-        assert exc_info.value.errno == errno.EBADF
-        assert exc_info.value.strerror == os.strerror(errno.EBADF)
+        if self.runappdirect:
+            # untranslated the errno can get reset by the calls to ll2ctypes
+            assert exc_info.value.errno == errno.EBADF
+            assert exc_info.value.strerror == os.strerror(errno.EBADF)
 
     def test_SetFromErrnoWithFilenameObject__PyList(self):
         import errno, os
@@ -318,8 +351,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename == [1, 2, "three"]
-        assert exc_info.value.errno == errno.EBADF
-        assert exc_info.value.strerror == os.strerror(errno.EBADF)
+        if self.runappdirect:
+            # untranslated the errno can get reset by the calls to ll2ctypes
+            assert exc_info.value.errno == errno.EBADF
+            assert exc_info.value.strerror == os.strerror(errno.EBADF)
 
     def test_SetFromErrnoWithFilenameObject__PyTuple(self):
         import errno, os
@@ -337,8 +372,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename == (1, 2, "three")
-        assert exc_info.value.errno == errno.EBADF
-        assert exc_info.value.strerror == os.strerror(errno.EBADF)
+        if self.runappdirect:
+            # untranslated the errno can get reset by the calls to ll2ctypes
+            assert exc_info.value.errno == errno.EBADF
+            assert exc_info.value.strerror == os.strerror(errno.EBADF)
 
     def test_SetFromErrnoWithFilenameObject__Py_None(self):
         import errno, os
@@ -356,8 +393,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename is None
-        assert exc_info.value.errno == errno.EBADF
-        assert exc_info.value.strerror == os.strerror(errno.EBADF)
+        if self.runappdirect:
+            # untranslated the errno can get reset by the calls to ll2ctypes
+            assert exc_info.value.errno == errno.EBADF
+            assert exc_info.value.strerror == os.strerror(errno.EBADF)
 
     def test_PyErr_Display(self):
         from sys import version_info
@@ -498,8 +537,10 @@ class AppTestFetch(AppTestCpythonExtensionBase):
 
              /* Proceed in thread 1 first */
              save = PyEval_SaveThread();
+             if (save == NULL) abort();
              while (id == 0 && flag == 0);
              gilsave = PyGILState_Ensure();
+             if (gilsave != PyGILState_UNLOCKED) abort();
 
              PyErr_Format(PyExc_ValueError, "%d", id);
 
