@@ -2,6 +2,7 @@ from rpython.rlib.unroll import unrolling_iterable
 from rpython.rlib import jit, rawrefcount
 from rpython.rlib.objectmodel import specialize, we_are_translated
 from rpython.rtyper.lltypesystem import rffi, lltype
+from rpython.rlib.rarithmetic import widen
 
 from pypy.interpreter.baseobjspace import DescrMismatch
 from pypy.interpreter.error import oefmt
@@ -263,6 +264,7 @@ def convert_member_defs(space, dict_w, members, w_type):
             dict_w[name] = w_descr
             i += 1
 
+WARN_MISSING_SLOTS = False
 missing_slots={}
 def warn_missing_slot(space, method_name, slot_name, w_type):
     if not we_are_translated():
@@ -289,7 +291,8 @@ def update_all_slots(space, w_type, pto):
 
         if not slot_func_helper:
             if not slot_apifunc:
-                warn_missing_slot(space, method_name, slot_name, w_type)
+                if WARN_MISSING_SLOTS:
+                    warn_missing_slot(space, method_name, slot_name, w_type)
                 continue
             slot_func_helper = slot_apifunc.get_llhelper(space)
         fill_slot(space, pto, w_type, slot_names, slot_func_helper)
@@ -381,7 +384,7 @@ def add_operators(space, dict_w, pto, name):
         dict_w[method_name] = w_obj
     if pto.c_tp_doc:
         raw_doc = rffi.charp2str(cts.cast('char*', pto.c_tp_doc))
-        dict_w['__doc__'] = space.newtext(extract_doc(raw_doc, name))
+        dict_w['__doc__'] = space.newtext_or_none(extract_doc(raw_doc, name))
     if pto.c_tp_new:
         add_tp_new_wrapper(space, dict_w, pto)
 
@@ -540,6 +543,13 @@ class W_PyCTypeObject(W_TypeObject):
         convert_getset_defs(space, dict_w, pto.c_tp_getset, self)
         convert_member_defs(space, dict_w, pto.c_tp_members, self)
 
+        w_dict = from_ref(space, pto.c_tp_dict)
+        if w_dict is not None:
+            dictkeys_w = space.listview(w_dict)
+            for w_key in dictkeys_w:
+                key = space.text_w(w_key)
+                dict_w[key] = space.getitem(w_dict, w_key)
+
         flag_heaptype = pto.c_tp_flags & Py_TPFLAGS_HEAPTYPE
         if flag_heaptype:
             minsize = rffi.sizeof(PyHeapTypeObject.TO)
@@ -786,8 +796,10 @@ def _type_realize(space, py_obj):
         # While this is a hack, cpython does it as well.
         w_metatype = space.w_type
 
-    w_obj = space.allocate_instance(W_PyCTypeObject, w_metatype)
-    track_reference(space, py_obj, w_obj)
+    w_obj = rawrefcount.to_obj(W_PyCTypeObject, py_obj)
+    if w_obj is None:
+        w_obj = space.allocate_instance(W_PyCTypeObject, w_metatype)
+        track_reference(space, py_obj, w_obj)
     # __init__ wraps all slotdefs functions from py_type via add_operators
     w_obj.__init__(space, py_type)
     w_obj.ready()
@@ -811,7 +823,7 @@ def finish_type_1(space, pto, bases_w=None):
     Sets up tp_bases, necessary before creating the interpreter type.
     """
     base = pto.c_tp_base
-    base_pyo = rffi.cast(PyObject, pto.c_tp_base)
+    base_pyo = rffi.cast(PyObject, base)
     if base and not base.c_tp_flags & Py_TPFLAGS_READY:
         type_realize(space, base_pyo)
     if base and not pto.c_ob_type: # will be filled later
@@ -897,6 +909,11 @@ def fill_ht_slot(ht, slotnum, ptr):
         if num == slotnum:
             setattr(getattr(ht, membername), slotname, rffi.cast(TARGET, ptr))
 
+def get_ht_slot(ht, slotnum):
+    for num, membername, slotname, TARGET in SLOT_TABLE:
+        if num == slotnum:
+            return rffi.cast(rffi.VOIDP, getattr(getattr(ht, membername), slotname))
+    return rffi.cast(rffi.VOIDP, 0)
 
 @cts.decl("""PyObject *
     PyType_FromSpecWithBases(PyType_Spec *spec, PyObject *bases)""",
@@ -979,6 +996,17 @@ def PyType_FromSpecWithBases(space, spec, bases):
     py_type_ready(space, typ)
     return cts.cast('PyObject*', res)
 
+
+@cpython_api([PyTypeObjectPtr, rffi.INT], rffi.VOIDP)
+def PyType_GetSlot(space, typ, slot):
+    """ Use the Py_tp* macros in typeslots.h to return a slot function
+    """
+    slot = widen(slot)
+    if slot < 0 or not typ.c_tp_flags & Py_TPFLAGS_HEAPTYPE:
+        raise oefmt(space.w_SystemError, "Bad internal call!")
+    heapobj = rffi.cast(PyHeapTypeObject, typ)
+    return get_ht_slot(heapobj, slot)
+
 @cpython_api([PyTypeObjectPtr, PyObject], PyObject, error=CANNOT_FAIL,
              result_borrowed=True)
 def _PyType_Lookup(space, type, w_name):
