@@ -31,6 +31,8 @@ def excepthook(exctype, value, traceback):
             # one is counting from the top, the other from the bottom of the
             # stack. so reverse polarity here
             if limit > 0:
+                if limit > sys.maxsize:
+                    limit = sys.maxsize
                 print_exception(exctype, value, traceback, limit=-limit)
             else:
                 # the limit is 0 or negative. PyTraceBack_Print does not print
@@ -68,6 +70,31 @@ def excepthook_failsafe(exctype, value):
     except:
         return False    # got an exception again... ignore, report the original
 
+def breakpointhook(*args, **kwargs):
+    """This hook function is called by built-in breakpoint()."""
+
+    import importlib, os, warnings
+
+    hookname = os.getenv('PYTHONBREAKPOINT')
+    if hookname is None or len(hookname) == 0:
+        hookname = 'pdb.set_trace'
+    elif hookname == '0':
+        return None
+    modname, dot, funcname = hookname.rpartition('.')
+    if dot == '':
+        modname = 'builtins'
+
+    try:
+        module = importlib.import_module(modname)
+        hook = getattr(module, funcname)
+    except:
+        warnings.warn(
+            'Ignoring unimportable $PYTHONBREAKPOINT: "{}"'.format(hookname),
+            RuntimeWarning)
+        return None
+
+    return hook(*args, **kwargs)
+
 def exit(exitcode=None):
     """Exit the interpreter by raising SystemExit(exitcode).
 If the exitcode is omitted or None, it defaults to zero (i.e., success).
@@ -75,7 +102,7 @@ If the exitcode is numeric, it will be used as the system exit status.
 If it is another kind of object, it will be printed and the system
 exit status will be one (i.e., failure)."""
     # note that we cannot simply use SystemExit(exitcode) here.
-    # in the default branch, we use "raise SystemExit, exitcode", 
+    # in the default branch, we use "raise SystemExit, exitcode",
     # which leads to an extra de-tupelizing
     # in normalize_exception, which is exactly like CPython's.
     if isinstance(exitcode, tuple):
@@ -109,7 +136,6 @@ All Rights Reserved.
 
 # This is tested in test_app_main.py
 class sysflags(metaclass=structseqtype):
-
     name = "sys.flags"
 
     debug = structseqfield(0)
@@ -125,34 +151,15 @@ class sysflags(metaclass=structseqtype):
     quiet = structseqfield(10)
     hash_randomization = structseqfield(11)
     isolated = structseqfield(12)
+    dev_mode = structseqfield(13)
+    utf8_mode = structseqfield(14)
 
-null_sysflags = sysflags((0,)*13)
+# no clue why dev_mode in particular has to be a bool, but CPython has tests
+# for that
+null_sysflags = sysflags((0,)*13 + (False, 0))
 null__xoptions = {}
 
 
-class asyncgen_hooks(metaclass=structseqtype):
-    name = "asyncgen_hooks"
-
-    firstiter = structseqfield(0)
-    finalizer = structseqfield(1)
-
-# FIXME: make this thread-local
-_current_asyncgen_hooks = asyncgen_hooks((None, None))
-
-def get_asyncgen_hooks():
-    return _current_asyncgen_hooks
-
-_default_arg = object()
-
-def set_asyncgen_hooks(firstiter=_default_arg, finalizer=_default_arg):
-    global _current_asyncgen_hooks
-    if firstiter is _default_arg:
-        firstiter = _current_asyncgen_hooks.firstiter
-    if finalizer is _default_arg:
-        finalizer = _current_asyncgen_hooks.finalizer
-    _current_asyncgen_hooks = asyncgen_hooks((firstiter, finalizer))
-
-
 implementation = SimpleNamespace(
     name='pypy',
     version=sys.version_info,
