@@ -75,6 +75,7 @@ class AppTestAppSysTests:
         builtin2 = sys.modules['builtins']
         assert builtins is builtin2, ( "import builtins "
                                        "is not sys.modules[builtins].")
+
     def test_builtin_module_names(self):
         import sys
         names = sys.builtin_module_names
@@ -103,28 +104,6 @@ class AppTestAppSysTests:
         assert exc_val2 ==e2
         assert tb2.tb_lineno - tb.tb_lineno == 6
 
-    def test_dynamic_attributes(self):
-        try:
-            raise Exception
-        except Exception as exc:
-            e = exc
-            import sys
-            exc_type = sys.exc_type
-            exc_val = sys.exc_value
-            tb = sys.exc_traceback
-        try:
-            raise Exception   # 8 lines below the previous one
-        except Exception as exc:
-            e2 = exc
-            exc_type2 = sys.exc_type
-            exc_val2 = sys.exc_value
-            tb2 = sys.exc_traceback
-        assert exc_type ==Exception
-        assert exc_val ==e
-        assert exc_type2 ==Exception
-        assert exc_val2 ==e2
-        assert tb2.tb_lineno - tb.tb_lineno == 8
-
     def test_exc_info_normalization(self):
         import sys
         try:
@@ -156,7 +135,13 @@ class AppTestAppSysTests:
 
     def test_getfilesystemencoding(self):
         import sys
-        assert sys.getfilesystemencoding() == self.filesystemenc
+        enc = sys.getfilesystemencoding()
+        if self.appdirect:
+            assert enc == self.filesystemenc
+        else:
+            # see comment in 'setup_after_space_initialization'
+            untranslated_enc = {'win32': 'mbcs', 'darwin': 'utf-8'}.get(enc, 'ascii')
+            assert enc == untranslated_enc
 
     def test_float_info(self):
         import sys
@@ -224,6 +209,10 @@ class AppTestAppSysTests:
         assert isinstance(info.lock, (str, type(None)))
         assert isinstance(info.version, (str, type(None)))
 
+    def test_sys_flags_dev_mode_is_bool(self):
+        import sys
+        assert type(sys.flags.dev_mode) is bool
+
 
 class AppTestSysModulePortedFromCPython:
     spaceconfig = {
@@ -517,6 +506,16 @@ class AppTestSysModulePortedFromCPython:
         sys.setrecursionlimit(oldlimit)
         raises(OverflowError, sys.setrecursionlimit, 1<<31)
 
+    def test_recursionlimit_toolow(self):
+        import sys
+        def callatlevel(l):
+            if l > 0:
+                callatlevel(l - 1)
+            else:
+                sys.setrecursionlimit(1)
+        with raises(RecursionError):
+            callatlevel(500)
+
     def test_getwindowsversion(self):
         import sys
         if hasattr(sys, "getwindowsversion"):
@@ -628,6 +627,7 @@ class AppTestSysModulePortedFromCPython:
         assert isinstance(vi[2], int)
         assert vi[3] in ("alpha", "beta", "candidate", "final")
         assert isinstance(vi[4], int)
+        assert isinstance(sys._framework, str)
 
     def test_implementation(self):
         import sys
@@ -804,6 +804,20 @@ class AppTestSysModulePortedFromCPython:
         assert cur.firstiter is None
         assert cur.finalizer is None
 
+    def test_coroutine_origin_tracking_depth(self):
+        import sys
+        depth = sys.get_coroutine_origin_tracking_depth()
+        assert depth == 0
+        try:
+            sys.set_coroutine_origin_tracking_depth(6)
+            depth = sys.get_coroutine_origin_tracking_depth()
+            assert depth == 6
+            with raises(ValueError):
+                sys.set_coroutine_origin_tracking_depth(-5)
+        finally:
+            sys.set_coroutine_origin_tracking_depth(0)
+
+
 
 class AppTestSysSettracePortedFromCpython(object):
     def test_sys_settrace(self):
