@@ -127,8 +127,14 @@ class Object(object):
     def descr__init__(space, w_obj, __args__):
         pass
 
+def get_printable_location(itergreenkey, w_itemtype):
+    return "DescrOperation.contains [%s, %s]" % (
+            itergreenkey.iterator_greenkey_printable(),
+            w_itemtype.getname(w_itemtype.space))
+
 contains_jitdriver = jit.JitDriver(name='contains',
-        greens=['w_type'], reds='auto')
+        greens=['itergreenkey', 'w_itemtype'], reds='auto',
+        get_printable_location=get_printable_location)
 
 class DescrOperation(object):
     # This is meant to be a *mixin*.
@@ -148,6 +154,8 @@ class DescrOperation(object):
     def get_and_call_function(space, w_descr, w_obj, *args_w):
         typ = type(w_descr)
         # a special case for performance and to avoid infinite recursion
+        # (possibly; but note issue3255 in the get() metehod, which might
+        # also remove the infinite recursion here)
         if typ is Function or typ is FunctionWithFixedCode:
             # isinstance(typ, Function) would not be correct here:
             # for a BuiltinFunction we must not use that shortcut, because a
@@ -181,7 +189,9 @@ class DescrOperation(object):
             return w_descr
         if w_type is None:
             w_type = space.type(w_obj)
-        return space.get_and_call_function(w_get, w_descr, w_obj, w_type)
+        # special case: don't use get_and_call_function() here.
+        # see test_issue3255 in apptest_descriptor.py
+        return space.call_function(w_get, w_descr, w_obj, w_type)
 
     def set(space, w_descr, w_obj, w_val):
         w_set = space.lookup(w_descr, '__set__')
@@ -267,9 +277,12 @@ class DescrOperation(object):
 
     def _check_len_result(space, w_obj):
         # Will complain if result is too big.
-        result = space.getindex_w(w_obj, space.w_OverflowError)
-        if result < 0:
+        w_result = space.index(w_obj)
+        assert space.isinstance_w(w_result, space.w_int)
+        if space.is_true(space.lt(w_result, space.newint(0))):
             raise oefmt(space.w_ValueError, "__len__() should return >= 0")
+        result = space.getindex_w(w_result, space.w_OverflowError)
+        assert result >= 0
         return result
 
     def is_iterable(space, w_obj):
@@ -305,6 +318,8 @@ class DescrOperation(object):
 
     def getitem(space, w_obj, w_key):
         w_descr = space.lookup(w_obj, '__getitem__')
+        if w_descr is None and space.isinstance_w(w_obj, space.w_type):
+            w_descr = space.getattr(w_obj, space.newtext('__class_getitem__'))
         if w_descr is None:
             raise oefmt(space.w_TypeError,
                         "'%T' object is not subscriptable (key %R)",
@@ -388,9 +403,10 @@ class DescrOperation(object):
 
     def sequence_contains(space, w_container, w_item):
         w_iter = space.iter(w_container)
-        w_type = space.type(w_iter)
+        itergreenkey = space.iterator_greenkey(w_iter)
+        w_itemtype = space.type(w_item)
         while 1:
-            contains_jitdriver.jit_merge_point(w_type=w_type)
+            contains_jitdriver.jit_merge_point(itergreenkey=itergreenkey, w_itemtype=w_itemtype)
             try:
                 w_next = space.next(w_iter)
             except OperationError as e:
@@ -438,7 +454,7 @@ class DescrOperation(object):
             return default_identity_hash(space, w_obj)
         if space.is_w(w_hash, space.w_None):
             raise oefmt(space.w_TypeError,
-                        "'%T' objects are unhashable", w_obj)
+                        "unhashable type: '%T'", w_obj)
         w_result = space.get_and_call_function(w_hash, w_obj)
         if not space.isinstance_w(w_result, space.w_int):
             raise oefmt(space.w_TypeError,
@@ -506,6 +522,10 @@ def _invoke_binop(space, w_impl, w_obj1, w_obj2):
             return w_res
     return None
 
+class PrintCache(object):
+    def __init__(self, space):
+        self.w_print = space.getattr(space.builtin, space.newtext("print"))
+
 
 # regular methods def helpers
 
@@ -515,6 +535,12 @@ def _make_binop_impl(symbol, specialnames):
         symbol.replace('%', '%%'),)
     seq_bug_compat = (symbol == '+' or symbol == '*')
 
+    printerrormsg = None
+    if symbol == ">>":
+        printerrormsg = errormsg + '. Did you mean "print(<message>, file=<output_stream>)"?'
+    if symbol == "-":
+        printerrormsg = errormsg + '. Did you mean "print(<-number>)"?'
+
     def binop_impl(space, w_obj1, w_obj2):
         w_typ1 = space.type(w_obj1)
         w_typ2 = space.type(w_obj2)
@@ -549,6 +575,8 @@ def _make_binop_impl(symbol, specialnames):
         w_res = _invoke_binop(space, w_right_impl, w_obj2, w_obj1)
         if w_res is not None:
             return w_res
+        if printerrormsg is not None and w_obj1 is space.fromcache(PrintCache).w_print:
+            raise oefmt(space.w_TypeError, printerrormsg, w_typ1, w_typ2)
         raise oefmt(space.w_TypeError, errormsg, w_typ1, w_typ2)
 
     return func_with_new_name(binop_impl, "binop_%s_impl"%left.strip('_'))
@@ -581,6 +609,8 @@ def _make_comparison_impl(symbol, specialnames):
     left, right = specialnames
     op = getattr(operator, left)
     def comparison_impl(space, w_obj1, w_obj2):
+        w_orig_obj1 = w_obj1
+        w_orig_obj2 = w_obj2
         w_typ1 = space.type(w_obj1)
         w_typ2 = space.type(w_obj2)
         w_left_src, w_left_impl = space.lookup_in_type_where(w_typ1, left)
@@ -620,7 +650,7 @@ def _make_comparison_impl(symbol, specialnames):
         # if we arrived here, they are unorderable
         raise oefmt(space.w_TypeError,
                     "'%s' not supported between instances of '%T' and '%T'",
-                    symbol, w_obj1, w_obj2)
+                    symbol, w_orig_obj1, w_orig_obj2)
 
     return func_with_new_name(comparison_impl, 'comparison_%s_impl'%left.strip('_'))
 
