@@ -32,8 +32,10 @@ class W_AbstractBytesObject(W_Root):
         s2 = space.bytes_w(w_other)
         if len(s2) > 1:
             return s1 is s2
+        if len(s2) == 0:
+            return len(s1) == 0
         else:            # strings of len <= 1 are unique-ified
-            return s1 == s2
+            return len(s1) == 1 and s1[0] == s2[0]
 
     def immutable_unique_id(self, space):
         if self.user_overridden_class:
@@ -73,6 +75,12 @@ class W_AbstractBytesObject(W_Root):
     def descr_hash(self, space):
         """x.__hash__() <==> hash(x)"""
 
+    def descr_isascii(self, space):
+        """B.isascii() -> bool
+
+        Return true if the string is empty or all characters in the string are ASCII, false otherwise.
+        ASCII characters have code points in the range U+0000-U+007F."""
+
     def descr_iter(self, space):
         """x.__iter__() <==> iter(x)"""
 
@@ -601,6 +609,12 @@ class W_BytesObject(W_AbstractBytesObject):
         x -= (x == -1) # convert -1 to -2 without creating a bridge
         return space.newint(x)
 
+    def descr_isascii(self, space):
+        for i in self._value:
+            if ord(i) > 127:
+                return space.w_False
+        return space.w_True
+
     def descr_eq(self, space, w_other):
         if not isinstance(w_other, W_BytesObject):
             return space.w_NotImplemented
@@ -862,6 +876,7 @@ W_BytesObject.typedef = TypeDef(
     isspace = interpindirect2app(W_AbstractBytesObject.descr_isspace),
     istitle = interpindirect2app(W_AbstractBytesObject.descr_istitle),
     isupper = interpindirect2app(W_AbstractBytesObject.descr_isupper),
+    isascii = interpindirect2app(W_AbstractBytesObject.descr_isascii),
     join = interpindirect2app(W_AbstractBytesObject.descr_join),
     ljust = interpindirect2app(W_AbstractBytesObject.descr_ljust),
     rjust = interpindirect2app(W_AbstractBytesObject.descr_rjust),
