@@ -137,6 +137,7 @@ class AppTestBytesObject:
         assert bytes.fromhex("abcd") == b'\xab\xcd'
         assert b''.fromhex("abcd") == b'\xab\xcd'
         assert bytes.fromhex("ab cd  ef") == b'\xab\xcd\xef'
+        assert bytes.fromhex("\nab\tcd  \tef\t") == b'\xab\xcd\xef'
         raises(TypeError, bytes.fromhex, b"abcd")
         raises(TypeError, bytes.fromhex, True)
         raises(ValueError, bytes.fromhex, "hello world")
@@ -281,6 +282,15 @@ class AppTestBytesObject:
         assert b'aaaa'.capitalize() == b'Aaaa'
         assert b'AaAa'.capitalize() == b'Aaaa'
 
+    def test_isascii(self):
+        assert b"hello".isascii() is True
+        assert b"\x00\x7f".isascii() is True
+        assert b"\x80".isascii() is False
+        assert b"\x97".isascii() is False
+        assert b"\xff".isascii() is False
+        assert b"Hello World\x00".isascii() is True
+        assert b"Hello World\x80".isascii() is False
+
     def test_rjust(self):
         s = b"abc"
         assert s.rjust(2) == s
@@ -803,6 +813,10 @@ class AppTestBytesObject:
         assert b'hello'.decode('ascii') == 'hello'
         raises(UnicodeDecodeError, b'he\x97lo'.decode, 'ascii')
 
+    def test_decode_surrogatepass_issue_3132(self):
+        with raises(UnicodeDecodeError):
+            b"\xd8=a".decode("utf-16-be", "surrogatepass")
+
     def test_encode(self):
         assert 'hello'.encode() == b'hello'
         assert type('hello'.encode()) is bytes
@@ -1050,3 +1064,10 @@ class AppTestBytesObject:
         id_a = id(a)
         assert a is not str(a, 'latin1')
         assert id_a != id_b
+
+    def test_error_message_wrong_self(self):
+        e = raises(TypeError, bytes.upper, 42)
+        assert "bytes" in str(e.value)
+        if hasattr(bytes.upper, 'im_func'):
+            e = raises(TypeError, bytes.upper.im_func, 42)
+            assert "'bytes'" in str(e.value)
