@@ -1405,7 +1405,7 @@ class AppTestListObject(object):
 
     def test_unicode_bug_in_listview_utf8(self):
         l1 = list(u'\u1234\u2345')
-        assert l1 == [u'\u1234', '\u2345']
+        assert l1 == [u'\u1234', u'\u2345']
 
     def test_list_from_set(self):
         l = ['a']
@@ -1658,6 +1658,9 @@ class AppTestListObject(object):
             assert L3.index(0.0, i) == i
             assert L3.index(-0.0, i) == i
 
+    def test_list_new_pos_only(self):
+        with raises(TypeError):
+            list(sequence=[])
 
 class AppTestWithoutStrategies:
     spaceconfig = {"objspace.std.withliststrategies": False}
