@@ -71,7 +71,7 @@ class AppTestObject:
                 return (self._name,), dict(value=int(self))
         import copyreg
         for protocol in [2, 3, 4]:
-            assert NamedInt("Name", value=42).__reduce__(protocol) == (
+            assert NamedInt("Name", value=42).__reduce_ex__(protocol) == (
                 copyreg.__newobj_ex__,
                 (NamedInt, ('Name',), dict(value=42)),
                 dict(_name='Name'), None, None)
@@ -136,6 +136,14 @@ class AppTestObject:
             pass
         assert X().__reduce_ex__(2)[2] is None
 
+    def test_reduce_arguments(self):
+        # since python3.7 object.__reduce__ doesn't take an argument anymore
+        # (used to be proto), and __reduce_ex__ requires one
+        with raises(TypeError):
+            object().__reduce__(0)
+        with raises(TypeError):
+            object().__reduce_ex__()
+
     def test_default_format(self):
         class x(object):
             def __str__(self):
@@ -175,6 +183,43 @@ class AppTestObject:
 
         raises(TypeError, B)
 
+    def test_object_init_not_really_overridden(self):
+        class A(object):
+            def __new__(cls, value):
+                return object.__new__(cls)
+            __init__ = object.__init__     # see issue #3239
+        assert isinstance(A(1), A)
+
+    def test_object_new_not_really_overridden(self):
+        class A(object):
+            def __init__(self, value):
+                self.value = value
+            __new__ = object.__new__
+        assert A(42).value == 42
+
+    def test_object_init_cant_call_parent_with_args(self):
+        class A(object):
+            def __init__(self, value):
+                object.__init__(self, value)
+        raises(TypeError, A, 1)
+
+    def test_object_new_cant_call_parent_with_args(self):
+        class A(object):
+            def __new__(cls, value):
+                return object.__new__(cls, value)
+        raises(TypeError, A, 1)
+
+    def test_object_init_and_new_overridden(self):
+        class A(object):
+            def __new__(cls, value):
+                result = object.__new__(cls)
+                result.other_value = value + 1
+                return result
+            def __init__(self, value):
+                self.value = value
+        assert A(42).value == 42
+        assert A(42).other_value == 43
+
     def test_object_str(self):
         # obscure case: __str__() must delegate to __repr__() without adding
         # type checking on its own
@@ -366,6 +411,14 @@ class AppTestObject:
         object.__init_subclass__() # does not crash
         raises(TypeError, object.__init_subclass__, 1)
 
+    def test_better_error_init(self):
+        class A: pass
+
+        with raises(TypeError) as excinfo:
+            A(1)
+        assert "A() takes no arguments" in str(excinfo.value)
+
+
 def test_isinstance_shortcut():
     from pypy.objspace.std import objspace
     space = objspace.StdObjSpace()
